@@ -43,12 +43,11 @@
 GST_DEBUG_CATEGORY_EXTERN (cairo_debug);
 #define GST_CAT_DEFAULT cairo_debug
 
-static GstElementDetails cairo_text_overlay_details = {
-  "Text Overlay",
-  "Filter/Editor/Video",
-  "Adds text strings on top of a video buffer",
-  "David Schleef <ds@schleef.org>"
-};
+static const GstElementDetails cairo_text_overlay_details =
+GST_ELEMENT_DETAILS ("Text overlay",
+    "Filter/Editor/Video",
+    "Adds text strings on top of a video buffer",
+    "David Schleef <ds@schleef.org>");
 
 enum
 {
@@ -115,9 +114,10 @@ static void gst_text_overlay_font_init (GstCairoTextOverlay * overlay);
 #define I420_SIZE(w,h)     (I420_V_OFFSET(w,h)+(I420_V_ROWSTRIDE(w)*GST_ROUND_UP_2(h)/2))
 
 GST_BOILERPLATE (GstCairoTextOverlay, gst_text_overlay, GstElement,
-    GST_TYPE_ELEMENT)
+    GST_TYPE_ELEMENT);
 
-     static void gst_text_overlay_base_init (gpointer g_class)
+static void
+gst_text_overlay_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
@@ -211,8 +211,8 @@ gst_text_overlay_init (GstCairoTextOverlay * overlay,
 {
   /* video sink */
   overlay->video_sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&video_sink_template_factory), "video_sink");
+      gst_pad_new_from_static_template (&video_sink_template_factory,
+      "video_sink");
   gst_pad_set_getcaps_function (overlay->video_sinkpad,
       GST_DEBUG_FUNCPTR (gst_text_overlay_getcaps));
   gst_pad_set_setcaps_function (overlay->video_sinkpad,
@@ -221,8 +221,8 @@ gst_text_overlay_init (GstCairoTextOverlay * overlay,
 
   /* text sink */
   overlay->text_sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&text_sink_template_factory), "text_sink");
+      gst_pad_new_from_static_template (&text_sink_template_factory,
+      "text_sink");
   gst_pad_set_link_function (overlay->text_sinkpad,
       GST_DEBUG_FUNCPTR (gst_text_overlay_text_pad_linked));
   gst_pad_set_unlink_function (overlay->text_sinkpad,
@@ -231,8 +231,8 @@ gst_text_overlay_init (GstCairoTextOverlay * overlay,
 
   /* (video) source */
   overlay->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-      (&cairo_text_overlay_src_template_factory), "src");
+      gst_pad_new_from_static_template
+      (&cairo_text_overlay_src_template_factory, "src");
   gst_pad_set_getcaps_function (overlay->srcpad,
       GST_DEBUG_FUNCPTR (gst_text_overlay_getcaps));
   gst_element_add_pad (GST_ELEMENT (overlay), overlay->srcpad);
@@ -556,6 +556,8 @@ gst_text_overlay_setcaps (GstPad * pad, GstCaps * caps)
   overlay->fps_n = gst_value_get_fraction_numerator (fps);
   overlay->fps_d = gst_value_get_fraction_denominator (fps);
 
+  gst_object_unref (overlay);
+
   return ret;
 }
 
