@@ -107,7 +107,7 @@
 #define NTSC_WIDE_PAR_X         40
 #define NTSC_WIDE_PAR_Y         33
 
-GST_DEBUG_CATEGORY (dvdemux_debug);
+GST_DEBUG_CATEGORY_STATIC (dvdemux_debug);
 #define GST_CAT_DEFAULT dvdemux_debug
 
 static const GstElementDetails dvdemux_details =
@@ -306,6 +306,7 @@ gst_dvdemux_add_pads (GstDVDemux * dvdemux)
   gst_pad_set_event_function (dvdemux->videosrcpad,
       GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
   gst_pad_use_fixed_caps (dvdemux->videosrcpad);
+  gst_pad_set_active (dvdemux->videosrcpad, TRUE);
   gst_element_add_pad (GST_ELEMENT (dvdemux), dvdemux->videosrcpad);
 
   dvdemux->audiosrcpad =
@@ -317,6 +318,7 @@ gst_dvdemux_add_pads (GstDVDemux * dvdemux)
   gst_pad_set_event_function (dvdemux->audiosrcpad,
       GST_DEBUG_FUNCPTR (gst_dvdemux_handle_src_event));
   gst_pad_use_fixed_caps (dvdemux->audiosrcpad);
+  gst_pad_set_active (dvdemux->audiosrcpad, TRUE);
   gst_element_add_pad (GST_ELEMENT (dvdemux), dvdemux->audiosrcpad);
 
   gst_element_no_more_pads (GST_ELEMENT (dvdemux));
@@ -1071,15 +1073,17 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
     if ((stop = demux->time_segment.stop) == -1)
       stop = demux->time_segment.duration;
 
-    /* now send the newsegment */
-    GST_DEBUG_OBJECT (demux, "Sending newsegment from %" GST_TIME_FORMAT
-        " to %" GST_TIME_FORMAT, GST_TIME_ARGS (demux->time_segment.start),
-        GST_TIME_ARGS (stop));
+    GST_INFO_OBJECT (demux,
+        "Saving newsegment event to be sent in streaming thread");
 
-    gst_dvdemux_push_event (demux,
-        gst_event_new_new_segment (FALSE,
-            demux->time_segment.rate, demux->time_segment.format,
-            demux->time_segment.last_stop, stop, demux->time_segment.time));
+    if (demux->pending_segment)
+      gst_event_unref (demux->pending_segment);
+
+    demux->pending_segment = gst_event_new_new_segment (FALSE,
+        demux->time_segment.rate, demux->time_segment.format,
+        demux->time_segment.last_stop, stop, demux->time_segment.time);
+
+    demux->need_segment = FALSE;
   }
 
   demux->running = TRUE;
@@ -1397,10 +1401,15 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
     goto done;
   }
 
-  ret = GST_FLOW_OK;
   gst_segment_set_last_stop (&dvdemux->time_segment, GST_FORMAT_TIME, next_ts);
   dvdemux->frame_offset++;
 
+  /* check for the end of the segment */
+  if (dvdemux->time_segment.stop != -1 && next_ts > dvdemux->time_segment.stop)
+    ret = GST_FLOW_UNEXPECTED;
+  else
+    ret = GST_FLOW_OK;
+
 done:
   return ret;
 
@@ -1484,6 +1493,7 @@ gst_dvdemux_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstDVDemux *dvdemux;
   GstFlowReturn ret;
+  GstClockTime timestamp;
 
   dvdemux = GST_DVDEMUX (gst_pad_get_parent (pad));
 
@@ -1493,6 +1503,14 @@ gst_dvdemux_chain (GstPad * pad, GstBuffer * buffer)
   if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT)))
     gst_adapter_clear (dvdemux->adapter);
 
+  /* a timestamp always should be respected */
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
+    gst_segment_set_last_stop (&dvdemux->time_segment, GST_FORMAT_TIME,
+        timestamp);
+    /* FIXME, adjust frame_offset and other counters */
+  }
+
   /* temporary hack? Can't do this from the state change */
   if (G_UNLIKELY (!dvdemux->videosrcpad))
     gst_dvdemux_add_pads (dvdemux);
@@ -1547,7 +1565,7 @@ gst_dvdemux_loop (GstPad * pad)
 
     /* check buffer size, don't want to read small buffers */
     if (G_UNLIKELY (GST_BUFFER_SIZE (buffer) < NTSC_BUFFER))
-      goto pause;
+      goto small_buffer;
 
     data = GST_BUFFER_DATA (buffer);
 
@@ -1603,21 +1621,29 @@ gst_dvdemux_loop (GstPad * pad)
       }
     }
   }
+
+
+  if (G_UNLIKELY (dvdemux->pending_segment)) {
+
+    /* now send the newsegment */
+    GST_DEBUG_OBJECT (dvdemux, "Sending newsegment from");
+
+    gst_dvdemux_push_event (dvdemux, dvdemux->pending_segment);
+    dvdemux->pending_segment = NULL;
+  }
+
   if (G_LIKELY (buffer == NULL)) {
     GST_DEBUG_OBJECT (dvdemux, "pulling buffer at offset %" G_GINT64_FORMAT,
         dvdemux->byte_segment.last_stop);
 
     ret = gst_pad_pull_range (dvdemux->sinkpad,
         dvdemux->byte_segment.last_stop, dvdemux->frame_len, &buffer);
-    if (ret == GST_FLOW_UNEXPECTED)
-      goto eos;
-
     if (ret != GST_FLOW_OK)
       goto pause;
 
     /* check buffer size, don't want to read small buffers */
     if (GST_BUFFER_SIZE (buffer) < dvdemux->frame_len)
-      goto pause;
+      goto small_buffer;
   }
   /* and decode the buffer */
   ret = gst_dvdemux_demux_frame (dvdemux, buffer);
@@ -1627,6 +1653,7 @@ gst_dvdemux_loop (GstPad * pad)
   /* and position ourselves for the next buffer */
   dvdemux->byte_segment.last_stop += dvdemux->frame_len;
 
+done:
   gst_object_unref (dvdemux);
 
   return;
@@ -1637,30 +1664,46 @@ parse_header_error:
     GST_ELEMENT_ERROR (dvdemux, STREAM, DECODE,
         (NULL), ("Error parsing DV header"));
     gst_buffer_unref (buffer);
-    ret = GST_FLOW_ERROR;
-    /* will stop the task */
-    goto pause;
-  }
-eos:
-  {
-    GST_LOG_OBJECT (dvdemux, "got eos");
+    dvdemux->running = FALSE;
+    gst_pad_pause_task (dvdemux->sinkpad);
     gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
-    /* pause with non-fatal error */
-    ret = GST_FLOW_WRONG_STATE;
-    goto pause;
+    goto done;
+  }
+small_buffer:
+  {
+    GST_ELEMENT_ERROR (dvdemux, STREAM, DECODE,
+        (NULL), ("Error reading buffer"));
+    gst_buffer_unref (buffer);
+    dvdemux->running = FALSE;
+    gst_pad_pause_task (dvdemux->sinkpad);
+    gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
+    goto done;
   }
 pause:
   {
-    GST_LOG_OBJECT (dvdemux, "pausing task %d", ret);
+    GST_INFO_OBJECT (dvdemux, "pausing task, %s", gst_flow_get_name (ret));
     dvdemux->running = FALSE;
     gst_pad_pause_task (dvdemux->sinkpad);
-    if (GST_FLOW_IS_FATAL (ret)) {
-      /* for fatal errors we post an error message */
-      GST_ELEMENT_ERROR (dvdemux, STREAM, FAILED,
-          (NULL), ("streaming stopped, reason %s", gst_flow_get_name (ret)));
-      gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
+    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+      if (ret == GST_FLOW_UNEXPECTED) {
+        GST_LOG_OBJECT (dvdemux, "got eos");
+        /* perform EOS logic */
+        if (dvdemux->time_segment.flags & GST_SEEK_FLAG_SEGMENT) {
+          gst_element_post_message (GST_ELEMENT (dvdemux),
+              gst_message_new_segment_done (GST_OBJECT_CAST (dvdemux),
+                  dvdemux->time_segment.format,
+                  dvdemux->time_segment.last_stop));
+        } else {
+          gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
+        }
+      } else {
+        /* for fatal errors or not-linked we post an error message */
+        GST_ELEMENT_ERROR (dvdemux, STREAM, FAILED,
+            (NULL), ("streaming stopped, reason %s", gst_flow_get_name (ret)));
+        gst_dvdemux_push_event (dvdemux, gst_event_new_eos ());
+      }
     }
-    gst_object_unref (dvdemux);
+    goto done;
   }
 }
 
@@ -1749,6 +1792,9 @@ gst_dvdemux_change_state (GstElement * element, GstStateChange transition)
 
       event_p = &dvdemux->seek_event;
       gst_event_replace (event_p, NULL);
+      if (dvdemux->pending_segment)
+        gst_event_unref (dvdemux->pending_segment);
+      dvdemux->pending_segment = NULL;
       break;
     }
     default:
