@@ -28,6 +28,7 @@
 #include "gstgconfaudiosrc.h"
 
 static void gst_gconf_audio_src_dispose (GObject * object);
+static void gst_gconf_audio_src_finalize (GstGConfAudioSrc * src);
 static void cb_toggle_element (GConfClient * client,
     guint connection_id, GConfEntry * entry, gpointer data);
 static GstStateChangeReturn
@@ -45,7 +46,7 @@ gst_gconf_audio_src_base_init (gpointer klass)
       "Source/Audio",
       "Audio source embedding the GConf-settings for audio input",
       "Ronald Bultje <rbultje@ronald.bitfreak.net>");
-  GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
+  static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
       GST_PAD_SRC,
       GST_PAD_ALWAYS,
       GST_STATIC_CAPS_ANY);
@@ -62,6 +63,7 @@ gst_gconf_audio_src_class_init (GstGConfAudioSrcClass * klass)
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
 
   oklass->dispose = gst_gconf_audio_src_dispose;
+  oklass->finalize = (GObjectFinalizeFunc) gst_gconf_audio_src_finalize;
   eklass->change_state = gst_gconf_audio_src_change_state;
 }
 
@@ -122,12 +124,17 @@ gst_gconf_audio_src_dispose (GObject * object)
     src->client = NULL;
   }
 
-  g_free (src->gconf_str);
-  src->gconf_str = NULL;
-
   GST_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
 
+static void
+gst_gconf_audio_src_finalize (GstGConfAudioSrc * src)
+{
+  g_free (src->gconf_str);
+
+  GST_CALL_PARENT (G_OBJECT_CLASS, finalize, ((GObject *) (src)));
+}
+
 static gboolean
 do_toggle_element (GstGConfAudioSrc * src)
 {
@@ -151,6 +158,7 @@ do_toggle_element (GstGConfAudioSrc * src)
 
   if (cur >= GST_STATE_READY || next == GST_STATE_PAUSED) {
     GST_DEBUG_OBJECT (src, "already running, ignoring GConf change");
+    g_free (new_gconf_str);
     return TRUE;
   }
 
