@@ -27,6 +27,7 @@
 #include "gstgconfvideosink.h"
 
 static void gst_gconf_video_sink_dispose (GObject * object);
+static void gst_gconf_video_sink_finalize (GstGConfVideoSink * sink);
 static void cb_toggle_element (GConfClient * client,
     guint connection_id, GConfEntry * entry, gpointer data);
 static GstStateChangeReturn
@@ -39,13 +40,12 @@ static void
 gst_gconf_video_sink_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
-  GstElementDetails gst_gconf_video_sink_details = {
-    "GConf video sink",
-    "Sink/Video",
-    "Video sink embedding the GConf-settings for video output",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>"
-  };
-  GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
+  static const GstElementDetails gst_gconf_video_sink_details =
+      GST_ELEMENT_DETAILS ("GConf video sink",
+      "Sink/Video",
+      "Video sink embedding the GConf-settings for video output",
+      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+  static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
       GST_PAD_SINK,
       GST_PAD_ALWAYS,
       GST_STATIC_CAPS_ANY);
@@ -62,6 +62,7 @@ gst_gconf_video_sink_class_init (GstGConfVideoSinkClass * klass)
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
 
   oklass->dispose = gst_gconf_video_sink_dispose;
+  oklass->finalize = (GObjectFinalizeFunc) gst_gconf_video_sink_finalize;
   eklass->change_state = gst_gconf_video_sink_change_state;
 }
 
@@ -102,7 +103,7 @@ gst_gconf_video_sink_init (GstGConfVideoSink * sink,
   sink->client = gconf_client_get_default ();
   gconf_client_add_dir (sink->client, GST_GCONF_DIR,
       GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
-  gconf_client_notify_add (sink->client, 
+  gconf_client_notify_add (sink->client,
       GST_GCONF_DIR "/" GST_GCONF_VIDEOSINK_KEY,
       cb_toggle_element, sink, NULL, NULL);
 }
@@ -122,6 +123,14 @@ gst_gconf_video_sink_dispose (GObject * object)
   GST_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
 
+static void
+gst_gconf_video_sink_finalize (GstGConfVideoSink * sink)
+{
+  g_free (sink->gconf_str);
+
+  GST_CALL_PARENT (G_OBJECT_CLASS, finalize, ((GObject *) (sink)));
+}
+
 static gboolean
 do_toggle_element (GstGConfVideoSink * sink)
 {
@@ -132,7 +141,7 @@ do_toggle_element (GstGConfVideoSink * sink)
   new_gconf_str = gst_gconf_get_string (GST_GCONF_VIDEOSINK_KEY);
   if (new_gconf_str != NULL && sink->gconf_str != NULL &&
       (strlen (new_gconf_str) == 0 ||
-      strcmp (sink->gconf_str, new_gconf_str) == 0)) {
+          strcmp (sink->gconf_str, new_gconf_str) == 0)) {
     g_free (new_gconf_str);
     GST_DEBUG_OBJECT (sink, "GConf key was updated, but it didn't change");
     return TRUE;
@@ -149,11 +158,12 @@ do_toggle_element (GstGConfVideoSink * sink)
   if (cur > GST_STATE_READY || next == GST_STATE_PAUSED) {
     GST_DEBUG_OBJECT (sink,
         "Auto-sink is already running. Ignoring GConf change");
+    g_free (new_gconf_str);
     return TRUE;
   }
 
   GST_DEBUG_OBJECT (sink, "GConf key changed: '%s' to '%s'",
-     GST_STR_NULL (sink->gconf_str), GST_STR_NULL (new_gconf_str));
+      GST_STR_NULL (sink->gconf_str), GST_STR_NULL (new_gconf_str));
 
   g_free (sink->gconf_str);
   sink->gconf_str = new_gconf_str;
