@@ -36,7 +36,7 @@
 #define GST_RGB24_SIZE(width,height)  ((height)*GST_RGB24_ROWSTRIDE(width))
 
 
-GST_DEBUG_CATEGORY (pixbufscale_debug);
+GST_DEBUG_CATEGORY_STATIC (pixbufscale_debug);
 #define GST_CAT_DEFAULT pixbufscale_debug
 
 /* elementfactory information */
@@ -81,7 +81,7 @@ static GType
 gst_pixbufscale_method_get_type (void)
 {
   static GType pixbufscale_method_type = 0;
-  static GEnumValue pixbufscale_methods[] = {
+  static const GEnumValue pixbufscale_methods[] = {
     {GST_PIXBUFSCALE_NEAREST, "0", "Nearest Neighbour"},
     {GST_PIXBUFSCALE_TILES, "1", "Tiles"},
     {GST_PIXBUFSCALE_BILINEAR, "2", "Bilinear"},
@@ -122,9 +122,10 @@ static gboolean gst_pixbufscale_handle_src_event (GstPad * pad,
 static gboolean parse_caps (GstCaps * caps, gint * width, gint * height);
 
 GST_BOILERPLATE (GstPixbufScale, gst_pixbufscale, GstBaseTransform,
-    GST_TYPE_BASE_TRANSFORM)
+    GST_TYPE_BASE_TRANSFORM);
 
-     static void gst_pixbufscale_base_init (gpointer g_class)
+static void
+gst_pixbufscale_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
@@ -169,8 +170,10 @@ static void
 gst_pixbufscale_init (GstPixbufScale * pixbufscale,
     GstPixbufScaleClass * kclass)
 {
+  GstBaseTransform *trans;
+
   GST_DEBUG_OBJECT (pixbufscale, "_init");
-  GstBaseTransform *trans = GST_BASE_TRANSFORM (pixbufscale);
+  trans = GST_BASE_TRANSFORM (pixbufscale);
 
   gst_pad_set_event_function (trans->srcpad, gst_pixbufscale_handle_src_event);
 
@@ -250,7 +253,7 @@ gst_pixbufscale_transform_caps (GstBaseTransform * trans,
     gst_structure_remove_field (structure, "pixel-aspect-ratio");
   }
 
-  GST_DEBUG_OBJECT (trans, "returning caps: %", ret);
+  GST_DEBUG_OBJECT (trans, "returning caps: %" GST_PTR_FORMAT, ret);
   return ret;
 }
 
@@ -303,7 +306,7 @@ gst_pixbufscale_get_unit_size (GstBaseTransform * trans,
   GstPixbufScale *pixbufscale;
   gint width, height;
 
-  g_return_val_if_fail (size, FALSE);
+  g_assert (size);
 
   pixbufscale = GST_PIXBUFSCALE (trans);
 
