@@ -110,6 +110,10 @@ struct _GstSignalProcessorPad
   GstBuffer *pen;
 
   guint index;
+
+  /* these are only used for sink pads */
+  guint samples_avail;
+  gfloat *data;
 };
 
 static GType
@@ -196,6 +200,8 @@ gst_signal_processor_add_pad_from_template (GstSignalProcessor * self,
       GST_DEBUG_FUNCPTR (gst_signal_processor_setcaps));
 
   if (templ->direction == GST_PAD_SINK) {
+    GST_DEBUG ("added new sink pad");
+
     gst_pad_set_event_function (new,
         GST_DEBUG_FUNCPTR (gst_signal_processor_event));
     gst_pad_set_chain_function (new,
@@ -203,6 +209,8 @@ gst_signal_processor_add_pad_from_template (GstSignalProcessor * self,
     gst_pad_set_activatepush_function (new,
         GST_DEBUG_FUNCPTR (gst_signal_processor_sink_activate_push));
   } else {
+    GST_DEBUG ("added new src pad");
+
     gst_pad_set_getrange_function (new,
         GST_DEBUG_FUNCPTR (gst_signal_processor_getrange));
     gst_pad_set_activatepull_function (new,
@@ -228,10 +236,18 @@ gst_signal_processor_init (GstSignalProcessor * self,
     templates = templates->next;
   }
 
+  self->state = GST_SIGNAL_PROCESSOR_STATE_NULL;
+
   self->audio_in = g_new0 (gfloat *, klass->num_audio_in);
   self->control_in = g_new0 (gfloat, klass->num_control_in);
   self->audio_out = g_new0 (gfloat *, klass->num_audio_out);
   self->control_out = g_new0 (gfloat, klass->num_control_out);
+
+  /* init */
+  self->pending_in = klass->num_audio_in;
+  self->pending_out = 0;
+
+  self->sample_rate = 0;
 }
 
 static void
@@ -251,35 +267,172 @@ gst_signal_processor_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static gboolean
+gst_signal_processor_setup (GstSignalProcessor * self, guint sample_rate)
+{
+  GstSignalProcessorClass *klass;
+  gboolean ret = TRUE;
+
+  klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
+
+  GST_INFO_OBJECT (self, "setup()");
+
+  g_return_val_if_fail (self->state == GST_SIGNAL_PROCESSOR_STATE_NULL, FALSE);
+
+  if (klass->setup)
+    ret = klass->setup (self, sample_rate);
+
+  if (!ret)
+    goto setup_failed;
+
+  self->state = GST_SIGNAL_PROCESSOR_STATE_INITIALIZED;
+
+  return ret;
+
+setup_failed:
+  {
+    GST_INFO_OBJECT (self, "setup() failed at %u Hz", sample_rate);
+    return ret;
+  }
+}
+
+static gboolean
+gst_signal_processor_start (GstSignalProcessor * self)
+{
+  GstSignalProcessorClass *klass;
+  gboolean ret = TRUE;
+
+  klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
+
+  g_return_val_if_fail (self->state == GST_SIGNAL_PROCESSOR_STATE_INITIALIZED,
+      FALSE);
+
+  GST_INFO_OBJECT (self, "start()");
+
+  if (klass->start)
+    ret = klass->start (self);
+
+  if (!ret)
+    goto start_failed;
+
+  self->state = GST_SIGNAL_PROCESSOR_STATE_RUNNING;
+
+  return ret;
+
+start_failed:
+  {
+    GST_INFO_OBJECT (self, "start() failed");
+    return ret;
+  }
+}
+
+static void
+gst_signal_processor_stop (GstSignalProcessor * self)
+{
+  GstSignalProcessorClass *klass;
+  GstElement *elem;
+  GList *sinks;
+
+  klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
+  elem = GST_ELEMENT (self);
+
+  GST_INFO_OBJECT (self, "stop()");
+
+  g_return_if_fail (self->state == GST_SIGNAL_PROCESSOR_STATE_RUNNING);
+
+  if (klass->stop)
+    klass->stop (self);
+
+  for (sinks = elem->sinkpads; sinks; sinks = sinks->next)
+    /* force set_caps when going to RUNNING, see note in set_caps */
+    gst_pad_set_caps (GST_PAD (sinks->data), NULL);
+
+  /* should also flush our buffers perhaps? */
+
+  self->state = GST_SIGNAL_PROCESSOR_STATE_INITIALIZED;
+}
+
+static void
+gst_signal_processor_cleanup (GstSignalProcessor * self)
+{
+  GstSignalProcessorClass *klass;
+
+  klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
+
+  GST_INFO_OBJECT (self, "cleanup()");
+
+  g_return_if_fail (self->state == GST_SIGNAL_PROCESSOR_STATE_INITIALIZED);
+
+  if (klass->cleanup)
+    klass->cleanup (self);
+
+  self->state = GST_SIGNAL_PROCESSOR_STATE_NULL;
+}
+
 static gboolean
 gst_signal_processor_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstSignalProcessor *self;
-  GstSignalProcessorClass *klass;
 
-  self = GST_SIGNAL_PROCESSOR (GST_PAD_PARENT (pad));
-  klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
+  self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
 
-  if (caps != self->caps) {
+  /* the whole processor has one caps; if the sample rate changes, let subclass
+     implementations know */
+  if (!gst_caps_is_equal (caps, self->caps)) {
     GstStructure *s;
-    gint sample_rate, buffer_frames;
+    gint sample_rate;
+
+    GST_DEBUG_OBJECT (pad, "got caps %" GST_PTR_FORMAT, caps);
 
     s = gst_caps_get_structure (caps, 0);
-    if (!gst_structure_get_int (s, "rate", &sample_rate))
-      return FALSE;
-    if (!gst_structure_get_int (s, "buffer-frames", &buffer_frames))
-      return FALSE;
+    if (!gst_structure_get_int (s, "rate", &sample_rate)) {
+      GST_WARNING ("got no sample-rate");
+      goto impossible;
+    }
 
-    if (!klass->setup (self, sample_rate))
-      return FALSE;
+    GST_DEBUG_OBJECT (self, "Got rate=%d", sample_rate);
+
+    if (GST_SIGNAL_PROCESSOR_IS_RUNNING (self))
+      gst_signal_processor_stop (self);
+    if (GST_SIGNAL_PROCESSOR_IS_INITIALIZED (self))
+      gst_signal_processor_cleanup (self);
+
+    if (!gst_signal_processor_setup (self, sample_rate))
+      goto start_failed;
 
     self->sample_rate = sample_rate;
-    self->buffer_frames = buffer_frames;
+    gst_caps_replace (&self->caps, caps);
+  } else {
+    GST_DEBUG_OBJECT (self, "skipping, have caps already");
   }
 
-  /* FIXME: handle was_active, etc */
+  /* we use this method to manage the processor's state, hence the caps clearing
+     in stop(). so it can be that we enter here just to manage the processor's
+     state, to take it to RUNNING from already being INITIALIZED with the right
+     sample rate (e.g., when having gone PLAYING->READY->PLAYING). make sure
+     when we leave that the processor is RUNNING. */
+  if (!GST_SIGNAL_PROCESSOR_IS_INITIALIZED (self)
+      && !gst_signal_processor_setup (self, self->sample_rate))
+    goto start_failed;
+  if (!GST_SIGNAL_PROCESSOR_IS_RUNNING (self)
+      && !gst_signal_processor_start (self))
+    goto start_failed;
+
+  gst_object_unref (self);
 
   return TRUE;
+
+start_failed:
+  {
+    gst_object_unref (self);
+    return FALSE;
+  }
+impossible:
+  {
+    g_critical ("something impossible happened");
+    gst_object_unref (self);
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -289,83 +442,174 @@ gst_signal_processor_event (GstPad * pad, GstEvent * event)
   GstSignalProcessorClass *bclass;
   gboolean ret;
 
-  self = GST_SIGNAL_PROCESSOR (GST_PAD_PARENT (pad));
+  self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
   bclass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
 
+  /* this probably isn't the correct interface: what about return values, what
+     about overriding event_default */
   if (bclass->event)
     bclass->event (self, event);
 
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FLUSH_START:
+      break;
+    case GST_EVENT_FLUSH_STOP:
+      /* clear errors now */
+      self->flow_state = GST_FLOW_OK;
+      break;
+    default:
+      break;
+  }
   ret = gst_pad_event_default (pad, event);
 
+  gst_object_unref (self);
+
   return ret;
 }
 
+static guint
+gst_signal_processor_prepare (GstSignalProcessor * self)
+{
+  GstElement *elem = (GstElement *) self;
+  GstSignalProcessorClass *klass;
+  GList *sinks, *srcs;
+  guint samples_avail = G_MAXUINT;
+
+  klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
+
+  /* first, assign audio_in pointers, and determine the number of samples that
+   * we can process */
+  for (sinks = elem->sinkpads; sinks; sinks = sinks->next) {
+    GstSignalProcessorPad *sinkpad;
+
+    sinkpad = (GstSignalProcessorPad *) sinks->data;
+    g_assert (sinkpad->samples_avail > 0);
+    samples_avail = MIN (samples_avail, sinkpad->samples_avail);
+    self->audio_in[sinkpad->index] = sinkpad->data;
+  }
+
+  if (samples_avail == G_MAXUINT) {
+    /* we don't have any sink pads, just choose a size -- should fix this
+     * function to have a suggested number of samples in the case of
+     * gst_pad_pull_range */
+    samples_avail = 256;
+  }
+
+  /* now assign output buffers. we can avoid allocation by reusing input
+     buffers, but only if process() can work in place, and if the input buffer
+     is the exact size of the number of samples we are processing. */
+  sinks = elem->sinkpads;
+  srcs = elem->srcpads;
+  if (GST_SIGNAL_PROCESSOR_CLASS_CAN_PROCESS_IN_PLACE (klass)) {
+    while (sinks && srcs) {
+      GstSignalProcessorPad *sinkpad, *srcpad;
+
+      sinkpad = (GstSignalProcessorPad *) sinks->data;
+      srcpad = (GstSignalProcessorPad *) srcs->data;
+
+      if (GST_BUFFER_SIZE (sinkpad->pen) == samples_avail * sizeof (gfloat)) {
+        /* reusable, yay */
+        g_assert (sinkpad->samples_avail == samples_avail);
+        srcpad->pen = sinkpad->pen;
+        sinkpad->pen = NULL;
+        self->audio_out[srcpad->index] = sinkpad->data;
+        self->pending_out++;
+
+        srcs = srcs->next;
+      }
+
+      sinks = sinks->next;
+    }
+  }
+
+  /* now allocate for any remaining outputs */
+  while (srcs) {
+    GstSignalProcessorPad *srcpad;
+    GstFlowReturn ret;
+
+    srcpad = (GstSignalProcessorPad *) srcs->data;
+
+    ret =
+        gst_pad_alloc_buffer_and_set_caps (GST_PAD (srcpad), -1,
+        samples_avail * sizeof (gfloat), GST_PAD_CAPS (srcpad), &srcpad->pen);
+
+    if (ret != GST_FLOW_OK) {
+      self->flow_state = ret;
+      return 0;
+    } else {
+      self->audio_out[srcpad->index] = (gfloat *) GST_BUFFER_DATA (srcpad->pen);
+      self->pending_out++;
+    }
+
+    srcs = srcs->next;
+  }
+
+  return samples_avail;
+}
+
+static void
+gst_signal_processor_update_inputs (GstSignalProcessor * self, guint nprocessed)
+{
+  GstElement *elem = (GstElement *) self;
+  GList *sinks;
+
+  for (sinks = elem->sinkpads; sinks; sinks = sinks->next) {
+    GstSignalProcessorPad *sinkpad;
+
+    sinkpad = (GstSignalProcessorPad *) sinks->data;
+    g_assert (sinkpad->samples_avail >= nprocessed);
+
+    if (sinkpad->pen && sinkpad->samples_avail == nprocessed) {
+      /* used up this buffer, unpen */
+      gst_buffer_unref (sinkpad->pen);
+      sinkpad->pen = NULL;
+    }
+
+    if (!sinkpad->pen) {
+      /* this buffer was used up */
+      self->pending_in++;
+      sinkpad->data = NULL;
+      sinkpad->samples_avail = 0;
+    } else {
+      /* advance ->data pointers and decrement ->samples_avail, unreffing buffer
+         if no samples are left */
+      sinkpad->samples_avail -= nprocessed;
+      sinkpad->data += nprocessed;      /* gfloat* arithmetic */
+    }
+  }
+}
+
 static void
 gst_signal_processor_process (GstSignalProcessor * self)
 {
   GstElement *elem;
-  GList *l1, *l2;
   GstSignalProcessorClass *klass;
+  guint nframes;
 
   g_return_if_fail (self->pending_in == 0);
   g_return_if_fail (self->pending_out == 0);
 
   elem = GST_ELEMENT (self);
 
-  /* arrange the output buffers */
-  for (l1 = elem->sinkpads, l2 = elem->srcpads; l1 || l2;
-      l1 = l1->next, l2 = l2->next) {
-    GstSignalProcessorPad *srcpad, *sinkpad;
-
-    if (!l2) {
-      /* the output buffers have been covered, yay */
-      break;
-    } else if (!l1) {
-      /* need to alloc some output buffers */
-      for (; l2; l2 = l2->next) {
-        GstFlowReturn ret;
-
-        srcpad = (GstSignalProcessorPad *) l2->data;
-
-        ret =
-            gst_pad_alloc_buffer_and_set_caps (GST_PAD (srcpad), -1,
-            self->buffer_frames, GST_PAD_CAPS (srcpad), &srcpad->pen);
-
-        if (ret != GST_FLOW_OK) {
-          self->state = ret;
-          return;
-        } else {
-          self->audio_out[srcpad->index] =
-              (gfloat *) GST_BUFFER_DATA (srcpad->pen);
-          self->pending_out++;
-        }
-      }
-      break;
-    } else {
-      /* copy input to output */
-      sinkpad = (GstSignalProcessorPad *) l1->data;
-      srcpad = (GstSignalProcessorPad *) l2->data;
-
-      srcpad->pen = sinkpad->pen;
-      sinkpad->pen = NULL;
-      self->audio_out[srcpad->index] = (gfloat *) GST_BUFFER_DATA (srcpad->pen);
-      self->pending_out++;
-    }
-  }
+  nframes = gst_signal_processor_prepare (self);
+  if (G_UNLIKELY (nframes == 0))
+    goto flow_error;
 
   klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
 
-  klass->process (self, self->buffer_frames);
+  GST_LOG_OBJECT (self, "process(%u)", nframes);
 
-  /* free unneeded input buffers */
+  klass->process (self, nframes);
 
-  for (l1 = elem->sinkpads; l1; l1 = l1->next) {
-    GstSignalProcessorPad *sinkpad = (GstSignalProcessorPad *) l1->data;
+  gst_signal_processor_update_inputs (self, nframes);
 
-    if (sinkpad->pen) {
-      gst_buffer_unref (sinkpad->pen);
-      sinkpad->pen = NULL;
-    }
+  return;
+
+flow_error:
+  {
+    GST_WARNING ("gst_pad_alloc_buffer_and_set_caps() returned %d",
+        self->flow_state);
+    return;
   }
 }
 
@@ -375,23 +619,27 @@ gst_signal_processor_pen_buffer (GstSignalProcessor * self, GstPad * pad,
 {
   GstSignalProcessorPad *spad = (GstSignalProcessorPad *) pad;
 
-  if (spad->pen) {
-    g_critical ("Pad %s:%s already has penned buffer",
-        GST_DEBUG_PAD_NAME (pad));
-    gst_buffer_unref (buffer);
-    return;
-  }
+  if (spad->pen)
+    goto had_buffer;
 
   /* keep the reference */
   spad->pen = buffer;
-  self->audio_in[spad->index] = (gfloat *) GST_BUFFER_DATA (buffer);
+  spad->data = (gfloat *) GST_BUFFER_DATA (buffer);
+  spad->samples_avail = GST_BUFFER_SIZE (buffer) / sizeof (float);
 
   g_assert (self->pending_in != 0);
 
   self->pending_in--;
 
-  if (self->pending_in == 0) {
-    gst_signal_processor_process (self);
+  return;
+
+  /* ERRORS */
+had_buffer:
+  {
+    GST_WARNING ("Pad %s:%s already has penned buffer",
+        GST_DEBUG_PAD_NAME (pad));
+    gst_buffer_unref (buffer);
+    return;
   }
 }
 
@@ -399,8 +647,11 @@ static void
 gst_signal_processor_flush (GstSignalProcessor * self)
 {
   GList *pads;
+  GstSignalProcessorClass *klass;
 
-  pads = GST_ELEMENT (self)->pads;
+  klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
+
+  GST_INFO_OBJECT (self, "flush()");
 
   for (pads = GST_ELEMENT (self)->pads; pads; pads = pads->next) {
     GstSignalProcessorPad *spad = (GstSignalProcessorPad *) pads->data;
@@ -408,16 +659,21 @@ gst_signal_processor_flush (GstSignalProcessor * self)
     if (spad->pen) {
       gst_buffer_unref (spad->pen);
       spad->pen = NULL;
+      spad->data = NULL;
+      spad->samples_avail = 0;
     }
   }
+
+  self->pending_out = 0;
+  self->pending_in = klass->num_audio_in;
 }
 
 static void
-gst_signal_processor_do_pulls (GstSignalProcessor * self)
+gst_signal_processor_do_pulls (GstSignalProcessor * self, guint nframes)
 {
   GList *sinkpads;
 
-  /* not threadsafe atm */
+  /* FIXME: not threadsafe atm */
 
   sinkpads = GST_ELEMENT (self)->sinkpads;
 
@@ -432,15 +688,15 @@ gst_signal_processor_do_pulls (GstSignalProcessor * self)
       continue;
     }
 
-    ret = gst_pad_pull_range (GST_PAD (spad), -1, self->buffer_frames, &buf);
+    ret = gst_pad_pull_range (GST_PAD (spad), -1, nframes, &buf);
 
     if (ret != GST_FLOW_OK) {
-      self->state = ret;
       gst_signal_processor_flush (self);
+      self->flow_state = ret;
       return;
     } else if (!buf) {
       g_critical ("Pull failed to make a buffer!");
-      self->state = GST_FLOW_ERROR;
+      self->flow_state = GST_FLOW_ERROR;
       return;
     } else {
       gst_signal_processor_pen_buffer (self, GST_PAD (spad), buf);
@@ -449,7 +705,7 @@ gst_signal_processor_do_pulls (GstSignalProcessor * self)
 
   if (self->pending_in != 0) {
     g_critical ("Something wierd happened...");
-    self->state = GST_FLOW_ERROR;
+    self->flow_state = GST_FLOW_ERROR;
   } else {
     gst_signal_processor_process (self);
   }
@@ -461,9 +717,9 @@ gst_signal_processor_getrange (GstPad * pad, guint64 offset,
 {
   GstSignalProcessor *self;
   GstSignalProcessorPad *spad = (GstSignalProcessorPad *) pad;
-  GstFlowReturn ret;
+  GstFlowReturn ret = GST_FLOW_ERROR;
 
-  self = GST_SIGNAL_PROCESSOR (GST_PAD_PARENT (pad));
+  self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
 
   if (spad->pen) {
     *buffer = spad->pen;
@@ -472,10 +728,11 @@ gst_signal_processor_getrange (GstPad * pad, guint64 offset,
     self->pending_out--;
     ret = GST_FLOW_OK;
   } else {
-    gst_signal_processor_do_pulls (self);
+    gst_signal_processor_do_pulls (self, length);
     if (!spad->pen) {
+      /* this is an error condition */
       *buffer = NULL;
-      ret = self->state;
+      ret = self->flow_state;
     } else {
       *buffer = spad->pen;
       spad->pen = NULL;
@@ -484,6 +741,10 @@ gst_signal_processor_getrange (GstPad * pad, guint64 offset,
     }
   }
 
+  GST_DEBUG_OBJECT (self, "returns %s", gst_flow_get_name (ret));
+
+  gst_object_unref (self);
+
   return ret;
 }
 
@@ -499,6 +760,7 @@ gst_signal_processor_do_pushes (GstSignalProcessor * self)
   for (; srcpads; srcpads = srcpads->next) {
     GstSignalProcessorPad *spad = (GstSignalProcessorPad *) srcpads->data;
     GstFlowReturn ret = GST_FLOW_OK;
+    GstBuffer *buffer;
 
     if (!spad->pen) {
       g_warning ("Unexpectedly empty buffer pen for pad %s:%s",
@@ -506,14 +768,17 @@ gst_signal_processor_do_pushes (GstSignalProcessor * self)
       continue;
     }
 
-    ret = gst_pad_push (GST_PAD (spad), spad->pen);
+    /* take buffer from pen */
+    buffer = spad->pen;
+    spad->pen = NULL;
+
+    ret = gst_pad_push (GST_PAD (spad), buffer);
 
     if (ret != GST_FLOW_OK) {
-      self->state = ret;
       gst_signal_processor_flush (self);
+      self->flow_state = ret;
       return;
     } else {
-      spad->pen = NULL;
       g_assert (self->pending_out > 0);
       self->pending_out--;
     }
@@ -521,7 +786,7 @@ gst_signal_processor_do_pushes (GstSignalProcessor * self)
 
   if (self->pending_out != 0) {
     g_critical ("Something wierd happened...");
-    self->state = GST_FLOW_ERROR;
+    self->flow_state = GST_FLOW_ERROR;
   }
 }
 
@@ -530,7 +795,7 @@ gst_signal_processor_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstSignalProcessor *self;
 
-  self = GST_SIGNAL_PROCESSOR (GST_PAD_PARENT (pad));
+  self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
 
   gst_signal_processor_pen_buffer (self, pad, buffer);
 
@@ -540,7 +805,9 @@ gst_signal_processor_chain (GstPad * pad, GstBuffer * buffer)
     gst_signal_processor_do_pushes (self);
   }
 
-  return self->state;
+  gst_object_unref (self);
+
+  return self->flow_state;
 }
 
 static void
@@ -576,7 +843,7 @@ gst_signal_processor_sink_activate_push (GstPad * pad, gboolean active)
   GstSignalProcessor *self;
   GstSignalProcessorClass *bclass;
 
-  self = GST_SIGNAL_PROCESSOR (GST_OBJECT_PARENT (pad));
+  self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
   bclass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
 
   if (active) {
@@ -601,6 +868,10 @@ gst_signal_processor_sink_activate_push (GstPad * pad, gboolean active)
     }
   }
 
+  GST_DEBUG_OBJECT (self, "result : %d", result);
+
+  gst_object_unref (self);
+
   return result;
 }
 
@@ -611,7 +882,7 @@ gst_signal_processor_src_activate_pull (GstPad * pad, gboolean active)
   GstSignalProcessor *self;
   GstSignalProcessorClass *bclass;
 
-  self = GST_SIGNAL_PROCESSOR (GST_OBJECT_PARENT (pad));
+  self = GST_SIGNAL_PROCESSOR (gst_pad_get_parent (pad));
   bclass = GST_SIGNAL_PROCESSOR_GET_CLASS (self);
 
   if (active) {
@@ -645,6 +916,10 @@ gst_signal_processor_src_activate_pull (GstPad * pad, gboolean active)
     }
   }
 
+  GST_DEBUG_OBJECT (self, "result : %d", result);
+
+  gst_object_unref (self);
+
   return result;
 }
 
@@ -652,18 +927,16 @@ static GstStateChangeReturn
 gst_signal_processor_change_state (GstElement * element,
     GstStateChange transition)
 {
-  /* GstSignalProcessor *self;
-     GstSignalProcessorClass *klass; */
+  GstSignalProcessor *self;
   GstStateChangeReturn result;
 
-  /* self = GST_SIGNAL_PROCESSOR (element);
-     klass = GST_SIGNAL_PROCESSOR_GET_CLASS (self); */
-
+  self = GST_SIGNAL_PROCESSOR (element);
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      self->flow_state = GST_FLOW_OK;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -671,19 +944,33 @@ gst_signal_processor_change_state (GstElement * element,
       break;
   }
 
-  result = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  if ((result =
+          GST_ELEMENT_CLASS (parent_class)->change_state (element,
+              transition)) == GST_STATE_CHANGE_FAILURE)
+    goto failure;
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      if (GST_SIGNAL_PROCESSOR_IS_RUNNING (self))
+        gst_signal_processor_stop (self);
+      gst_signal_processor_flush (self);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      /* gst_signal_processor_cleanup (self); */
+      if (GST_SIGNAL_PROCESSOR_IS_INITIALIZED (self))
+        gst_signal_processor_cleanup (self);
       break;
     default:
       break;
   }
 
   return result;
+
+  /* ERRORS */
+failure:
+  {
+    GST_DEBUG_OBJECT (element, "parent failed state change");
+    return result;
+  }
 }
