@@ -28,9 +28,9 @@ G_BEGIN_DECLS
 
 #define GST_TYPE_PNGDEC (gst_pngdec_get_type())
 #define GST_PNGDEC(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_PNGDEC,GstPngDec))
-#define GST_PNGDEC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_PNGDEC,GstPngDec))
+#define GST_PNGDEC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_PNGDEC,GstPngDecClass))
 #define GST_IS_PNGDEC(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_PNGDEC))
-#define GST_IS_PNGDEC_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_PNGDEC))
+#define GST_IS_PNGDEC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_PNGDEC))
 
 typedef struct _GstPngDec GstPngDec;
 typedef struct _GstPngDecClass GstPngDecClass;
@@ -67,6 +67,8 @@ struct _GstPngDec
   gboolean framed;
   GstClockTime in_timestamp;
   GstClockTime in_duration;
+
+  GstSegment segment;
 };
 
 struct _GstPngDecClass
