@@ -842,6 +842,8 @@ no_handle:
   }
 no_ports:
   {
+    raw1394_destroy_handle (src->handle);
+    src->handle = NULL;
     GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL),
         ("no ports available for raw1394"));
     return FALSE;
@@ -854,6 +856,12 @@ cannot_set_port:
   }
 cannot_start:
   {
+    raw1394_destroy_handle (src->handle);
+    src->handle = NULL;
+#ifdef HAVE_LIBIEC61883
+    iec61883_dv_fb_close (src->iec61883dv);
+    src->iec61883dv = NULL;
+#endif
     GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
         ("can't start 1394 iso receive"));
     return FALSE;
@@ -861,6 +869,8 @@ cannot_start:
 #ifdef HAVE_LIBIEC61883
 cannot_initialise_dv:
   {
+    raw1394_destroy_handle (src->handle);
+    src->handle = NULL;
     GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
         ("can't initialise iec61883 dv"));
     return FALSE;
@@ -1074,7 +1084,7 @@ gst_dv1394src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
   g_free (protocol);
 
   location = gst_uri_get_location (uri);
-  if (location && (!(location == "")))
+  if (location && *location != '\0')
     gst_dv1394src->port = strtol (location, NULL, 10);
   else
     gst_dv1394src->port = DEFAULT_PORT;
