@@ -221,7 +221,9 @@ speex_dec_convert (GstPad * pad,
         case GST_FORMAT_BYTES:
           scale = sizeof (float) * dec->header->nb_channels;
         case GST_FORMAT_DEFAULT:
-          *dest_value = scale * (src_value * dec->header->rate / GST_SECOND);
+          *dest_value =
+              gst_util_uint64_scale_int (scale * src_value, dec->header->rate,
+              GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -234,7 +236,9 @@ speex_dec_convert (GstPad * pad,
           *dest_value = src_value * sizeof (float) * dec->header->nb_channels;
           break;
         case GST_FORMAT_TIME:
-          *dest_value = src_value * GST_SECOND / dec->header->rate;
+          *dest_value =
+              gst_util_uint64_scale_int (src_value, GST_SECOND,
+              dec->header->rate);
           break;
         default:
           res = FALSE;
@@ -247,8 +251,8 @@ speex_dec_convert (GstPad * pad,
           *dest_value = src_value / (sizeof (float) * dec->header->nb_channels);
           break;
         case GST_FORMAT_TIME:
-          *dest_value = src_value * GST_SECOND /
-              (dec->header->rate * sizeof (float) * dec->header->nb_channels);
+          *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
+              dec->header->rate * sizeof (float) * dec->header->nb_channels);
           break;
         default:
           res = FALSE;
@@ -440,12 +444,12 @@ speex_dec_sink_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:{
       GstFormat format;
-      gdouble rate;
+      gdouble rate, arate;
       gint64 start, stop, time;
       gboolean update;
 
-      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
-          &stop, &time);
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
+          &start, &stop, &time);
 
       if (format != GST_FORMAT_TIME)
         goto newseg_wrong_format;
@@ -454,8 +458,8 @@ speex_dec_sink_event (GstPad * pad, GstEvent * event)
         goto newseg_wrong_rate;
 
       /* now configure the values */
-      gst_segment_set_newsegment (&dec->segment, update,
-          rate, GST_FORMAT_TIME, start, stop, time);
+      gst_segment_set_newsegment_full (&dec->segment, update,
+          rate, arate, GST_FORMAT_TIME, start, stop, time);
 
       dec->granulepos = -1;
 
@@ -500,29 +504,18 @@ speex_dec_chain_parse_header (GstSpeexDec * dec, GstBuffer * buf)
   dec->header = speex_packet_to_header ((char *) GST_BUFFER_DATA (buf),
       GST_BUFFER_SIZE (buf));
 
-  if (!dec->header) {
-    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
-        (NULL), ("couldn't read header"));
-    return GST_FLOW_ERROR;
-  }
+  if (!dec->header)
+    goto no_header;
 
-  if (dec->header->mode >= SPEEX_NB_MODES) {
-    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
-        (NULL),
-        ("Mode number %d does not (yet/any longer) exist in this version",
-            dec->header->mode));
-    return GST_FLOW_ERROR;
-  }
+  if (dec->header->mode >= SPEEX_NB_MODES)
+    goto mode_too_old;
 
   dec->mode = (SpeexMode *) speex_mode_list[dec->header->mode];
 
   /* initialize the decoder */
   dec->state = speex_decoder_init (dec->mode);
-  if (!dec->state) {
-    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
-        (NULL), ("couldn't initialize decoder"));
-    return GST_FLOW_ERROR;
-  }
+  if (!dec->state)
+    goto init_failed;
 
   speex_decoder_ctl (dec->state, SPEEX_SET_ENH, &dec->enh);
   speex_decoder_ctl (dec->state, SPEEX_GET_FRAME_SIZE, &dec->frame_size);
@@ -549,13 +542,40 @@ speex_dec_chain_parse_header (GstSpeexDec * dec, GstBuffer * buf)
       "endianness", G_TYPE_INT, G_BYTE_ORDER,
       "width", G_TYPE_INT, 16, "depth", G_TYPE_INT, 16, NULL);
 
-  if (!gst_pad_set_caps (dec->srcpad, caps)) {
-    gst_caps_unref (caps);
-    return GST_FLOW_NOT_NEGOTIATED;
-  }
+  if (!gst_pad_set_caps (dec->srcpad, caps))
+    goto nego_failed;
 
   gst_caps_unref (caps);
   return GST_FLOW_OK;
+
+  /* ERRORS */
+no_header:
+  {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+        (NULL), ("couldn't read header"));
+    return GST_FLOW_ERROR;
+  }
+mode_too_old:
+  {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+        (NULL),
+        ("Mode number %d does not (yet/any longer) exist in this version",
+            dec->header->mode));
+    return GST_FLOW_ERROR;
+  }
+init_failed:
+  {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+        (NULL), ("couldn't initialize decoder"));
+    return GST_FLOW_ERROR;
+  }
+nego_failed:
+  {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+        (NULL), ("couldn't negotiate format"));
+    gst_caps_unref (caps);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 }
 
 static GstFlowReturn
@@ -564,8 +584,7 @@ speex_dec_chain_parse_comments (GstSpeexDec * dec, GstBuffer * buf)
   GstTagList *list;
   gchar *ver, *encoder = NULL;
 
-  list = gst_tag_list_from_vorbiscomment_buffer (buf,
-      (const guint8 *) "", 1, &encoder);
+  list = gst_tag_list_from_vorbiscomment_buffer (buf, NULL, 0, &encoder);
 
   if (!list) {
     GST_WARNING_OBJECT (dec, "couldn't decode comments");
@@ -607,18 +626,28 @@ static GstFlowReturn
 speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf)
 {
   GstFlowReturn res = GST_FLOW_OK;
-  gint i;
+  gint i, fpp;
+  guint size;
+  guint8 *data;
+
+  data = GST_BUFFER_DATA (buf);
+  size = GST_BUFFER_SIZE (buf);
 
   /* send data to the bitstream */
-  speex_bits_read_from (&dec->bits, (char *) GST_BUFFER_DATA (buf),
-      GST_BUFFER_SIZE (buf));
+  speex_bits_read_from (&dec->bits, (char *) data, size);
+
+  fpp = dec->header->frames_per_packet;
+
+  GST_DEBUG_OBJECT (dec, "received buffer of size %u, fpp %d", size, fpp);
 
   /* now decode each frame */
-  for (i = 0; i < dec->header->frames_per_packet; i++) {
+  for (i = 0; i < fpp; i++) {
     GstBuffer *outbuf;
     gint64 timestamp;
     gint16 *out_data;
-    gint ret;
+    gint ret, j;
+
+    GST_LOG_OBJECT (dec, "decoding frame %d/%d", i, fpp);
 
     ret = speex_decode (dec->state, &dec->bits, dec->output);
     if (ret == -1) {
@@ -629,6 +658,7 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf)
       GST_WARNING_OBJECT (dec, "Decoding error: corrupted stream?");
       break;
     }
+
     if (speex_bits_remaining (&dec->bits) < 0) {
       GST_WARNING_OBJECT (dec, "Decoding overflow: corrupted stream?");
       break;
@@ -648,13 +678,13 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf)
     out_data = (gint16 *) GST_BUFFER_DATA (outbuf);
 
     /*PCM saturation (just in case) */
-    for (i = 0; i < dec->frame_size * dec->header->nb_channels; i++) {
-      if (dec->output[i] > 32767.0)
-        out_data[i] = 32767;
+    for (j = 0; j < dec->frame_size * dec->header->nb_channels; j++) {
+      if (dec->output[j] > 32767.0)
+        out_data[j] = 32767;
       else if (dec->output[i] < -32768.0)
-        out_data[i] = -32768;
+        out_data[j] = -32768;
       else
-        out_data[i] = (gint16) dec->output[i];
+        out_data[j] = (gint16) dec->output[j];
     }
 
     if (dec->granulepos == -1) {
