@@ -107,6 +107,10 @@ struct _GstTagDemuxPrivate
   GstTagList *event_tags;
   GstTagList *parsed_tags;
   gboolean send_tag_event;
+
+  GstSegment segment;
+  gboolean need_newseg;
+  gboolean newseg_update;
 };
 
 /* Require at least 8kB of data before we attempt typefind. 
@@ -151,6 +155,7 @@ static gboolean gst_tag_demux_pad_query (GstPad * pad, GstQuery * query);
 static const GstQueryType *gst_tag_demux_get_query_types (GstPad * pad);
 static gboolean gst_tag_demux_get_upstream_size (GstTagDemux * tagdemux);
 static void gst_tag_demux_send_tag_event (GstTagDemux * tagdemux);
+static gboolean gst_tag_demux_send_new_segment (GstTagDemux * tagdemux);
 
 static void gst_tag_demux_base_init (gpointer g_class);
 static void gst_tag_demux_class_init (gpointer g_class, gpointer d);
@@ -177,9 +182,8 @@ gst_tag_demux_get_type (void)
       (GInstanceInitFunc) gst_tag_demux_init
     };
 
-    /* FIXME: register as 'GstApeDemuxBase' for now */
     object_type = g_type_register_static (GST_TYPE_ELEMENT,
-        "GstTagDemux", &object_info, G_TYPE_FLAG_ABSTRACT);
+        "GstApeDemuxBase", &object_info, G_TYPE_FLAG_ABSTRACT);
   }
 
   return object_type;
@@ -237,6 +241,10 @@ gst_tag_demux_reset (GstTagDemux * tagdemux)
     gst_tag_list_free (tagdemux->priv->parsed_tags);
     tagdemux->priv->parsed_tags = NULL;
   }
+
+  gst_segment_init (&tagdemux->priv->segment, GST_FORMAT_UNDEFINED);
+  tagdemux->priv->need_newseg = TRUE;
+  tagdemux->priv->newseg_update = FALSE;
 }
 
 static void
@@ -329,6 +337,7 @@ gst_tag_demux_add_srcpad (GstTagDemux * tagdemux, GstCaps * new_caps)
         tagdemux->priv->src_caps);
 
     gst_object_ref (tagdemux->priv->srcpad);
+    gst_pad_set_active (tagdemux->priv->srcpad, TRUE);
     if (!(gst_element_add_pad (GST_ELEMENT (tagdemux), tagdemux->priv->srcpad)))
       return FALSE;
     gst_element_no_more_pads (GST_ELEMENT (tagdemux));
@@ -353,6 +362,10 @@ gst_tag_demux_remove_srcpad (GstTagDemux * demux)
   return res;
 };
 
+/* will return FALSE if buffer is beyond end of data; will return TRUE
+ * if buffer was trimmed successfully or didn't need trimming, but may
+ * also return TRUE and set *buf_ref to NULL if the buffer was before
+ * the start of the data */
 static gboolean
 gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref)
 {
@@ -377,7 +390,7 @@ gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref)
 
       if (out_offset >= v1tag_offset) {
         GST_DEBUG_OBJECT (tagdemux, "Buffer is past the end of the data");
-        goto no_out_buffer;
+        goto no_out_buffer_end;
       }
 
       if (out_offset + out_size > v1tag_offset) {
@@ -392,7 +405,7 @@ gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref)
     if (out_offset <= tagdemux->priv->strip_start) {
       if (out_offset + out_size <= tagdemux->priv->strip_start) {
         GST_DEBUG_OBJECT (tagdemux, "Buffer is before the start of the data");
-        goto no_out_buffer;
+        goto no_out_buffer_start;
       }
 
       trim_start = tagdemux->priv->strip_start - out_offset;
@@ -431,10 +444,18 @@ gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref)
 
   return TRUE;
 
-no_out_buffer:
-  gst_buffer_unref (buf);
-  *buf_ref = NULL;
-  return TRUE;
+no_out_buffer_end:
+  {
+    gst_buffer_unref (buf);
+    *buf_ref = NULL;
+    return FALSE;
+  }
+no_out_buffer_start:
+  {
+    gst_buffer_unref (buf);
+    *buf_ref = NULL;
+    return TRUE;
+  }
 }
 
 static void
@@ -445,11 +466,13 @@ gst_tag_demux_chain_parse_tag (GstTagDemux * demux, GstBuffer * collect)
   guint tagsize = 0;
   guint available;
 
+  g_assert (gst_buffer_is_metadata_writable (collect));
+
   klass = GST_TAG_DEMUX_CLASS (G_OBJECT_GET_CLASS (demux));
 
   /* If we receive a buffer that's from the middle of the file, 
    * we can't read tags so move to typefinding */
-  if (GST_BUFFER_OFFSET (collect) != 0) {
+  if (GST_BUFFER_OFFSET_IS_VALID (collect) && GST_BUFFER_OFFSET (collect) != 0) {
     GST_DEBUG_OBJECT (demux, "Received buffer from non-zero offset %"
         G_GINT64_FORMAT ". Can't read tags", GST_BUFFER_OFFSET (collect));
     demux->priv->state = GST_TAG_DEMUX_TYPEFINDING;
@@ -473,6 +496,12 @@ gst_tag_demux_chain_parse_tag (GstTagDemux * demux, GstBuffer * collect)
     return;
   }
 
+  /* need to set offset of first buffer to 0 or trimming won't work */
+  if (!GST_BUFFER_OFFSET_IS_VALID (collect)) {
+    GST_WARNING_OBJECT (demux, "Fixing up first buffer without offset");
+    GST_BUFFER_OFFSET (collect) = 0;
+  }
+
   GST_DEBUG_OBJECT (demux, "Identified tag, size = %u bytes", tagsize);
 
   do {
@@ -528,6 +557,18 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
   demux = GST_TAG_DEMUX (GST_PAD_PARENT (pad));
   g_return_val_if_fail (GST_IS_TAG_DEMUX (demux), GST_FLOW_ERROR);
 
+  /* Update our segment last_stop info */
+  if (demux->priv->segment.format == GST_FORMAT_BYTES) {
+    if (GST_BUFFER_OFFSET_IS_VALID (buf))
+      demux->priv->segment.last_stop = GST_BUFFER_OFFSET (buf);
+    demux->priv->segment.last_stop += GST_BUFFER_SIZE (buf);
+  } else if (demux->priv->segment.format == GST_FORMAT_TIME) {
+    if (GST_BUFFER_TIMESTAMP_IS_VALID (buf))
+      demux->priv->segment.last_stop = GST_BUFFER_TIMESTAMP (buf);
+    if (GST_BUFFER_DURATION_IS_VALID (buf))
+      demux->priv->segment.last_stop += GST_BUFFER_DURATION (buf);
+  }
+
   if (demux->priv->collect == NULL) {
     demux->priv->collect = buf;
   } else {
@@ -537,6 +578,8 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
 
   switch (demux->priv->state) {
     case GST_TAG_DEMUX_READ_START_TAG:
+      demux->priv->collect =
+          gst_buffer_make_metadata_writable (demux->priv->collect);
       gst_tag_demux_chain_parse_tag (demux, demux->priv->collect);
       if (demux->priv->state != GST_TAG_DEMUX_TYPEFINDING)
         break;
@@ -557,7 +600,7 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
       typefind_buf = demux->priv->collect;
       gst_buffer_ref (typefind_buf);
       if (!gst_tag_demux_trim_buffer (demux, &typefind_buf))
-        return GST_FLOW_ERROR;
+        return GST_FLOW_UNEXPECTED;
 
       if (typefind_buf == NULL)
         break;                  /* Still need more data */
@@ -609,7 +652,7 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
         outbuf = demux->priv->collect;
         demux->priv->collect = NULL;
         if (!gst_tag_demux_trim_buffer (demux, &outbuf))
-          return GST_FLOW_ERROR;
+          return GST_FLOW_UNEXPECTED;
       }
       if (outbuf) {
         if (G_UNLIKELY (demux->priv->srcpad == NULL)) {
@@ -625,6 +668,15 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
         outbuf = gst_buffer_make_metadata_writable (outbuf);
         gst_buffer_set_caps (outbuf, GST_PAD_CAPS (demux->priv->srcpad));
 
+        /* Might need a new segment before the buffer */
+        if (demux->priv->need_newseg) {
+          if (!gst_tag_demux_send_new_segment (demux)) {
+            GST_DEBUG_OBJECT (demux, "Failed to send new segment event");
+            goto error;
+          }
+          demux->priv->need_newseg = FALSE;
+        }
+
         return gst_pad_push (demux->priv->srcpad, outbuf);
       }
     }
@@ -653,6 +705,22 @@ gst_tag_demux_sink_event (GstPad * pad, GstEvent * event)
       }
       ret = gst_pad_event_default (pad, event);
       break;
+    case GST_EVENT_NEWSEGMENT:{
+      gboolean update;
+      gdouble rate, arate;
+      GstFormat format;
+      gint64 start, stop, position;
+
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate,
+          &format, &start, &stop, &position);
+
+      gst_segment_set_newsegment_full (&demux->priv->segment, update, rate,
+          arate, format, start, stop, position);
+      demux->priv->newseg_update = update;
+      demux->priv->need_newseg = TRUE;
+      ret = TRUE;
+      break;
+    }
     default:
       ret = gst_pad_event_default (pad, event);
       break;
@@ -665,27 +733,21 @@ gst_tag_demux_sink_event (GstPad * pad, GstEvent * event)
 static gboolean
 gst_tag_demux_get_upstream_size (GstTagDemux * tagdemux)
 {
-  GstPad *peer = NULL;
   GstFormat format;
-  gint64 result;
-  gboolean res = FALSE;
+  gint64 len;
 
   /* Short-cut if we already queried upstream */
   if (tagdemux->priv->upstream_size > 0)
     return TRUE;
 
-  if ((peer = gst_pad_get_peer (tagdemux->priv->sinkpad)) == NULL)
-    return FALSE;
-
   format = GST_FORMAT_BYTES;
-  if (gst_pad_query_duration (peer, &format, &result) &&
-      format == GST_FORMAT_BYTES && result > 0) {
-    tagdemux->priv->upstream_size = result;
-    res = TRUE;
+  if (!gst_pad_query_peer_duration (tagdemux->priv->sinkpad, &format, &len) ||
+      len <= 0) {
+    return FALSE;
   }
 
-  gst_object_unref (peer);
-  return res;
+  tagdemux->priv->upstream_size = len;
+  return TRUE;
 }
 
 static gboolean
@@ -715,12 +777,18 @@ gst_tag_demux_srcpad_event (GstPad * pad, GstEvent * event)
 
         switch (cur_type) {
           case GST_SEEK_TYPE_SET:
+            if (cur == -1)
+              cur = 0;
             cur += tagdemux->priv->strip_start;
             break;
           case GST_SEEK_TYPE_CUR:
             break;
           case GST_SEEK_TYPE_END:
-            cur += tagdemux->priv->strip_end;
+            /* Adjust the seek to be relative to the start of any end tag
+             * (note: 10 bytes before end is represented by stop=-10) */
+            if (cur > 0)
+              cur = 0;
+            cur -= tagdemux->priv->strip_end;
             break;
           default:
             g_assert_not_reached ();
@@ -728,12 +796,19 @@ gst_tag_demux_srcpad_event (GstPad * pad, GstEvent * event)
         }
         switch (stop_type) {
           case GST_SEEK_TYPE_SET:
-            stop += tagdemux->priv->strip_start;
+            if (stop != -1) {
+              /* -1 means the end of the file, pass it upstream intact */
+              stop += tagdemux->priv->strip_start;
+            }
             break;
           case GST_SEEK_TYPE_CUR:
             break;
           case GST_SEEK_TYPE_END:
-            stop += tagdemux->priv->strip_end;
+            /* Adjust the seek to be relative to the start of any end tag
+             * (note: 10 bytes before end is represented by stop=-10) */
+            if (stop > 0)
+              stop = 0;
+            stop -= tagdemux->priv->strip_end;
             break;
           default:
             break;
@@ -1164,26 +1239,37 @@ gst_tag_demux_read_range (GstTagDemux * demux,
   if (!gst_tag_demux_get_upstream_size (demux))
     return GST_FLOW_ERROR;
 
-  if (in_offset + length >= demux->priv->upstream_size - demux->priv->strip_end)
+  if (in_offset + length >= demux->priv->upstream_size - demux->priv->strip_end) {
+    if (in_offset + demux->priv->strip_end >= demux->priv->upstream_size)
+      return GST_FLOW_UNEXPECTED;
     in_length = demux->priv->upstream_size - demux->priv->strip_end - in_offset;
-  else
+  } else {
     in_length = length;
+  }
 
   ret = gst_pad_pull_range (demux->priv->sinkpad, in_offset, in_length, buffer);
 
   if (ret == GST_FLOW_OK && *buffer) {
     if (!gst_tag_demux_trim_buffer (demux, buffer))
-      goto error;
+      goto read_beyond_end;
+
+    /* this should only happen in streaming mode */
+    g_assert (*buffer != NULL);
+
+    gst_buffer_set_caps (*buffer, demux->priv->src_caps);
   }
 
   return ret;
 
-error:
-  if (*buffer != NULL) {
-    gst_buffer_unref (buffer);
-    *buffer = NULL;
+read_beyond_end:
+  {
+    GST_DEBUG_OBJECT (demux, "attempted read beyond end of file");
+    if (*buffer != NULL) {
+      gst_buffer_unref (buffer);
+      *buffer = NULL;
+    }
+    return GST_FLOW_UNEXPECTED;
   }
-  return GST_FLOW_ERROR;
 }
 
 static GstFlowReturn
@@ -1303,3 +1389,83 @@ gst_tag_demux_send_tag_event (GstTagDemux * demux)
     gst_pad_push_event (demux->priv->srcpad, event);
   }
 }
+
+static gboolean
+gst_tag_demux_send_new_segment (GstTagDemux * tagdemux)
+{
+  GstEvent *event;
+  gint64 start, stop, position;
+  GstSegment *seg = &tagdemux->priv->segment;
+
+  if (seg->format == GST_FORMAT_UNDEFINED) {
+    GST_LOG_OBJECT (tagdemux,
+        "No new segment received before first buffer. Using default");
+    gst_segment_set_newsegment (seg, FALSE, 1.0,
+        GST_FORMAT_BYTES, tagdemux->priv->strip_start, -1,
+        tagdemux->priv->strip_start);
+  }
+
+  /* Can't adjust segments in non-BYTES formats */
+  if (tagdemux->priv->segment.format != GST_FORMAT_BYTES) {
+    event = gst_event_new_new_segment_full (tagdemux->priv->newseg_update,
+        seg->rate, seg->applied_rate, seg->format, seg->start,
+        seg->stop, seg->time);
+    return gst_pad_push_event (tagdemux->priv->srcpad, event);
+  }
+
+  start = seg->start;
+  stop = seg->stop;
+  position = seg->time;
+
+  g_return_val_if_fail (start != -1, FALSE);
+  g_return_val_if_fail (position != -1, FALSE);
+
+  if (tagdemux->priv->strip_end > 0) {
+    if (gst_tag_demux_get_upstream_size (tagdemux)) {
+      guint64 v1tag_offset =
+          tagdemux->priv->upstream_size - tagdemux->priv->strip_end;
+
+      if (start >= v1tag_offset) {
+        /* Segment is completely within the ID3v1 tag, output an open-ended
+         * segment, even though all the buffers will get trimmed away */
+        start = v1tag_offset;
+        stop = -1;
+      }
+
+      if (stop != -1 && stop >= v1tag_offset) {
+        GST_DEBUG_OBJECT (tagdemux,
+            "Segment crosses the ID3v1 tag. Trimming end");
+        stop = v1tag_offset;
+      }
+    }
+  }
+
+  if (tagdemux->priv->strip_start > 0) {
+    if (start > tagdemux->priv->strip_start)
+      start -= tagdemux->priv->strip_start;
+    else
+      start = 0;
+
+    if (position > tagdemux->priv->strip_start)
+      position -= tagdemux->priv->strip_start;
+    else
+      position = 0;
+
+    if (stop != -1) {
+      if (stop > tagdemux->priv->strip_start)
+        stop -= tagdemux->priv->strip_start;
+      else
+        stop = 0;
+    }
+  }
+
+  GST_DEBUG_OBJECT (tagdemux,
+      "Sending new segment update %d, rate %g, format %d, "
+      "start %" G_GINT64_FORMAT ", stop %" G_GINT64_FORMAT ", position %"
+      G_GINT64_FORMAT, tagdemux->priv->newseg_update, seg->rate, seg->format,
+      start, stop, position);
+
+  event = gst_event_new_new_segment_full (tagdemux->priv->newseg_update,
+      seg->rate, seg->applied_rate, seg->format, start, stop, position);
+  return gst_pad_push_event (tagdemux->priv->srcpad, event);
+}
