@@ -21,7 +21,7 @@
 /**
  * SECTION:element-auparse
  * @short_description: .au file parser
- * 
+ *
  * <refsect2>
  * <para>
  * Parses .au files.
@@ -158,7 +158,6 @@ gst_au_parse_reset (GstAuParse * auparse)
   auparse->encoding = 0;
   auparse->samplerate = 0;
   auparse->channels = 0;
-  auparse->float_swap = 0;
 
   gst_adapter_clear (auparse->adapter);
 
@@ -269,6 +268,12 @@ gst_au_parse_parse_header (GstAuParse * auparse)
   auparse->samplerate = GST_READ_UINT32_BE (head + 16);
   auparse->channels = GST_READ_UINT32_BE (head + 20);
 
+  if (auparse->samplerate < 8000 || auparse->samplerate > 192000)
+    goto unsupported_sample_rate;
+
+  if (auparse->channels < 1 || auparse->channels > 2)
+    goto unsupported_number_of_channels;
+
   GST_DEBUG_OBJECT (auparse, "offset %" G_GINT64_FORMAT ", size %u, "
       "encoding %u, frequency %u, channels %u", auparse->offset, size,
       auparse->encoding, auparse->samplerate, auparse->channels);
@@ -283,8 +288,6 @@ gst_au_parse_parse_header (GstAuParse * auparse)
    * http://www.tsp.ece.mcgill.ca/MMSP/Documents/AudioFormats/AU/Samples.html
    */
 
-  auparse->float_swap = 0;
-
   switch (auparse->encoding) {
     case 1:                    /* 8-bit ISDN mu-law G.711 */
       law = 1;
@@ -363,13 +366,9 @@ gst_au_parse_parse_header (GstAuParse * auparse)
     tempcaps = gst_caps_new_simple ("audio/x-raw-float",
         "rate", G_TYPE_INT, auparse->samplerate,
         "channels", G_TYPE_INT, auparse->channels,
-        "endianness", G_TYPE_INT, G_BYTE_ORDER,
+        "endianness", G_TYPE_INT, auparse->endianness,
         "width", G_TYPE_INT, depth, NULL);
     auparse->sample_size = auparse->channels * depth / 8;
-    if (auparse->endianness != G_BYTE_ORDER) {
-      GST_DEBUG_OBJECT (auparse, "need to swap float byte order ourselves!");
-      auparse->float_swap = depth;
-    }
   } else if (layout[0]) {
     tempcaps = gst_caps_new_simple ("audio/x-adpcm",
         "layout", G_TYPE_STRING, layout, NULL);
@@ -402,9 +401,22 @@ unknown_header:
     GST_ELEMENT_ERROR (auparse, STREAM, WRONG_TYPE, (NULL), (NULL));
     return GST_FLOW_ERROR;
   }
+unsupported_sample_rate:
+  {
+    GST_ELEMENT_ERROR (auparse, STREAM, FORMAT, (NULL),
+        ("Unsupported samplerate: %u", auparse->samplerate));
+    return GST_FLOW_ERROR;
+  }
+unsupported_number_of_channels:
+  {
+    GST_ELEMENT_ERROR (auparse, STREAM, FORMAT, (NULL),
+        ("Unsupported number of channels: %u", auparse->channels));
+    return GST_FLOW_ERROR;
+  }
 unknown_format:
   {
-    GST_ELEMENT_ERROR (auparse, STREAM, FORMAT, (NULL), (NULL));
+    GST_ELEMENT_ERROR (auparse, STREAM, FORMAT, (NULL),
+        ("Unsupported encoding: %u", auparse->encoding));
     return GST_FLOW_ERROR;
   }
 add_pad_failed:
@@ -474,35 +486,7 @@ gst_au_parse_chain (GstPad * pad, GstBuffer * buf)
     }
 
     data = gst_adapter_peek (auparse->adapter, sendnow);
-
-    /* audioconvert only handles floats in native endianness ... */
-    switch (auparse->float_swap) {
-      case 32:{
-        guint32 *indata = (guint32 *) data;
-        guint32 *outdata = (guint32 *) GST_BUFFER_DATA (outbuf);
-        gint i;
-
-        for (i = 0; i < (sendnow / sizeof (guint32)); ++i) {
-          outdata[i] = GUINT32_SWAP_LE_BE (indata[i]);
-        }
-        break;
-      }
-      case 64:{
-        guint64 *indata = (guint64 *) data;
-        guint64 *outdata = (guint64 *) GST_BUFFER_DATA (outbuf);
-        gint i;
-
-        for (i = 0; i < (sendnow / sizeof (guint64)); ++i) {
-          outdata[i] = GUINT64_SWAP_LE_BE (indata[i]);
-        }
-        break;
-      }
-      default:{
-        memcpy (GST_BUFFER_DATA (outbuf), data, sendnow);
-        break;
-      }
-    }
-
+    memcpy (GST_BUFFER_DATA (outbuf), data, sendnow);
     gst_adapter_flush (auparse->adapter, sendnow);
 
     auparse->buffer_offset += sendnow;
