@@ -28,7 +28,7 @@
 
 #include <gst/video/video.h>
 
-GST_DEBUG_CATEGORY (navigationtest_debug);
+GST_DEBUG_CATEGORY_STATIC (navigationtest_debug);
 #define GST_CAT_DEFAULT navigationtest_debug
 
 static const GstElementDetails navigationtest_details =
