@@ -39,7 +39,7 @@
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_EDGETV,GstEdgeTVClass))
 #define GST_IS_EDGETV(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_EDGETV))
-#define GST_IS_EDGETV_CLASS(obj) \
+#define GST_IS_EDGETV_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_EDGETV))
 
 typedef struct _GstEdgeTV GstEdgeTV;
