@@ -26,14 +26,20 @@
  *
  * <refsect2>
  * <para>
- * icydemux accepts data streams with ICY metadata at known intervals, as transmitted from an upstream element (usually read as response headers from an HTTP stream). The mime type of the data between the tag blocks is detected using typefind functions, and the appropriate output mime type set on outgoing buffers. 
+ * icydemux accepts data streams with ICY metadata at known intervals, as
+ * transmitted from an upstream element (usually read as response headers from
+ * an HTTP stream). The mime type of the data between the tag blocks is
+ * detected using typefind functions, and the appropriate output mime type set
+ * on outgoing buffers. 
  * </para>
  * <title>Example launch line</title>
  * <para>
  * <programlisting>
  * gst-launch gnomevfssrc location=http://some.server/ ! icydemux ! fakesink -t
  * </programlisting>
- * This pipeline should read any available ICY tag information and output it. The contents of the stream should be detected, and the appropriate mime type set on buffers produced from icydemux.
+ * This pipeline should read any available ICY tag information and output it.
+ * The contents of the stream should be detected, and the appropriate mime
+ * type set on buffers produced from icydemux.
  * </para>
  * </refsect2>
  */
@@ -42,6 +48,7 @@
 #endif
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
+#include <gst/tag/tag.h>
 
 #include "gsticydemux.h"
 
@@ -56,7 +63,7 @@ GST_ELEMENT_DETAILS ("ICY tag demuxer",
 
 #define ICY_TYPE_FIND_MAX_SIZE (40*1024)
 
-GST_DEBUG_CATEGORY (icydemux_debug);
+GST_DEBUG_CATEGORY_STATIC (icydemux_debug);
 #define GST_CAT_DEFAULT (icydemux_debug)
 
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -177,10 +184,9 @@ gst_icydemux_reset (GstICYDemux * icydemux)
     icydemux->meta_adapter = NULL;
   }
 
-  if (icydemux->typefind_adapter) {
-    gst_adapter_clear (icydemux->typefind_adapter);
-    g_object_unref (icydemux->typefind_adapter);
-    icydemux->typefind_adapter = NULL;
+  if (icydemux->typefind_buf) {
+    gst_buffer_unref (icydemux->typefind_buf);
+    icydemux->typefind_buf = NULL;
   }
 }
 
@@ -263,6 +269,7 @@ gst_icydemux_add_srcpad (GstICYDemux * icydemux, GstCaps * new_caps)
     GST_DEBUG_OBJECT (icydemux, "Adding src pad with caps %" GST_PTR_FORMAT,
         icydemux->src_caps);
 
+    gst_pad_set_active (icydemux->srcpad, TRUE);
     if (!(gst_element_add_pad (GST_ELEMENT (icydemux), icydemux->srcpad)))
       return FALSE;
     gst_element_no_more_pads (GST_ELEMENT (icydemux));
@@ -285,40 +292,14 @@ gst_icydemux_remove_srcpad (GstICYDemux * icydemux)
   return res;
 };
 
-/* The following two charset mangling functions were copied from gnomevfssrc.
- * Preserve them under the unverified assumption that they do something vaguely
- * worthwhile.
- */
-static char *
-unicodify (const char *str, int len, ...)
+static gchar *
+gst_icydemux_unicodify (const gchar * str)
 {
-  char *ret = NULL, *cset;
-  va_list args;
-  gsize bytes_read, bytes_written;
+  const gchar *env_vars[] = { "GST_ICY_TAG_ENCODING",
+    "GST_TAG_ENCODING", NULL
+  };
 
-  if (g_utf8_validate (str, len, NULL))
-    return g_strndup (str, len >= 0 ? len : strlen (str));
-
-  va_start (args, len);
-  while ((cset = va_arg (args, char *)) != NULL)
-  {
-    if (!strcmp (cset, "locale"))
-      ret = g_locale_to_utf8 (str, len, &bytes_read, &bytes_written, NULL);
-    else
-      ret = g_convert (str, len, "UTF-8", cset,
-          &bytes_read, &bytes_written, NULL);
-    if (ret)
-      break;
-  }
-  va_end (args);
-
-  return ret;
-}
-
-static char *
-gst_icydemux_unicodify (const char *str)
-{
-  return unicodify (str, -1, "locale", "ISO-8859-1", NULL);
+  return gst_tag_freeform_string_to_utf8 (str, -1, env_vars);
 }
 
 static void
@@ -427,49 +408,38 @@ static GstFlowReturn
 gst_icydemux_typefind_or_forward (GstICYDemux * icydemux, GstBuffer * buf)
 {
   if (icydemux->typefinding) {
-    GstBuffer *typefind_buf;
-    const guint8 *data;
+    GstBuffer *tf_buf;
     GstCaps *caps;
-    int size;
-    guint prob;
+    GstTypeFindProbability prob;
 
-    if (!icydemux->typefind_adapter)
-      icydemux->typefind_adapter = gst_adapter_new ();
-
-    gst_adapter_push (icydemux->typefind_adapter, buf);
-
-    size = gst_adapter_available (icydemux->typefind_adapter);
-    typefind_buf = gst_buffer_new ();
-
-    data = gst_adapter_peek (icydemux->typefind_adapter, size);
-
-    GST_BUFFER_DATA (typefind_buf) = (guint8 *) data;
-    GST_BUFFER_SIZE (typefind_buf) = size;
+    if (icydemux->typefind_buf) {
+      icydemux->typefind_buf = gst_buffer_join (icydemux->typefind_buf, buf);
+    } else {
+      icydemux->typefind_buf = buf;
+    }
 
     caps = gst_type_find_helper_for_buffer (GST_OBJECT (icydemux),
-        typefind_buf, &prob);
+        icydemux->typefind_buf, &prob);
 
     if (caps == NULL) {
-      if (size < ICY_TYPE_FIND_MAX_SIZE) {
-        gst_buffer_unref (typefind_buf);
+      if (GST_BUFFER_SIZE (icydemux->typefind_buf) < ICY_TYPE_FIND_MAX_SIZE) {
         /* Just break for more data */
         return GST_FLOW_OK;
       }
 
       /* We failed typefind */
-      GST_ELEMENT_ERROR (icydemux, CORE, CAPS,
-          ("Could not determine the mime type of the file"),
+      GST_ELEMENT_ERROR (icydemux, STREAM, TYPE_NOT_FOUND, (NULL),
           ("No caps found for contents within an ICY stream"));
-      gst_buffer_unref (typefind_buf);
-      gst_adapter_clear (icydemux->typefind_adapter);
+      gst_buffer_unref (icydemux->typefind_buf);
+      icydemux->typefind_buf = NULL;
       return GST_FLOW_ERROR;
     }
 
     if (!gst_icydemux_add_srcpad (icydemux, caps)) {
       GST_DEBUG_OBJECT (icydemux, "Failed to add srcpad");
       gst_caps_unref (caps);
-      gst_buffer_unref (typefind_buf);
-      gst_adapter_clear (icydemux->typefind_adapter);
+      gst_buffer_unref (icydemux->typefind_buf);
+      icydemux->typefind_buf = NULL;
       return GST_FLOW_ERROR;
     }
     gst_caps_unref (caps);
@@ -487,21 +457,23 @@ gst_icydemux_typefind_or_forward (GstICYDemux * icydemux, GstBuffer * buf)
      * to get that forwarded. */
     icydemux->typefinding = FALSE;
 
-    data = gst_adapter_take (icydemux->typefind_adapter, size);
-    GST_BUFFER_DATA (typefind_buf) = (guint8 *) data;
-    GST_BUFFER_MALLOCDATA (typefind_buf) = (guint8 *) data;
-
-    return gst_icydemux_typefind_or_forward (icydemux, typefind_buf);
+    tf_buf = icydemux->typefind_buf;
+    icydemux->typefind_buf = NULL;
+    return gst_icydemux_typefind_or_forward (icydemux, tf_buf);
   } else {
     if (G_UNLIKELY (icydemux->srcpad == NULL)) {
       gst_buffer_unref (buf);
       return GST_FLOW_ERROR;
     }
 
+    buf = gst_buffer_make_metadata_writable (buf);
     gst_buffer_set_caps (buf, icydemux->src_caps);
 
-    /* Most things don't care, and it's a pain to track */
-    GST_BUFFER_OFFSET (buf) = GST_BUFFER_OFFSET_NONE;
+    /* Most things don't care, and it's a pain to track (we should preserve a
+     * 0 offset on the first buffer though if it's there, for id3demux etc.) */
+    if (GST_BUFFER_OFFSET (buf) != 0) {
+      GST_BUFFER_OFFSET (buf) = GST_BUFFER_OFFSET_NONE;
+    }
 
     return gst_pad_push (icydemux->srcpad, buf);
   }
