@@ -242,7 +242,6 @@ const struct ID3v2FrameIDConvert
 } frame_id_conversions[] = {
   /* 2.3.x frames */
   {
-  "TDAT", "TDRC"}, {
   "TORY", "TDOR"}, {
   "TYER", "TDRC"},
       /* 2.2.x frames */
@@ -266,7 +265,7 @@ const struct ID3v2FrameIDConvert
   "TCM", "TCOM"}, {
   "TCO", "TCON"}, {
   "TCR", "TCOP"}, {
-  "TDA", "TDRC"}, {
+  "TDA", "TDAT"}, {             /* obsolete, but we need to parse it anyway */
   "TDY", "TDLY"}, {
   "TEN", "TENC"}, {
   "TFT", "TFLT"}, {
@@ -329,11 +328,56 @@ convert_fid_to_v240 (gchar * frame_id)
   return FALSE;
 }
 
+
+/* add unknown or unhandled ID3v2 frames to the taglist as binary blobs */
+static void
+id3demux_add_id3v2_frame_blob_to_taglist (ID3TagsWorking * work, guint size)
+{
+  GstBuffer *blob;
+  GstCaps *caps;
+  guint8 *frame_data;
+  gchar *media_type;
+  guint frame_size, header_size;
+
+  switch (ID3V2_VER_MAJOR (work->hdr.version)) {
+    case 1:
+    case 2:
+      header_size = 3 + 3;
+      break;
+    case 3:
+    case 4:
+      header_size = 4 + 4 + 2;
+      break;
+    default:
+      g_return_if_reached ();
+  }
+
+  frame_data = work->hdr.frame_data - header_size;
+  frame_size = size + header_size;
+
+  blob = gst_buffer_new_and_alloc (frame_size);
+  memcpy (GST_BUFFER_DATA (blob), frame_data, frame_size);
+
+  media_type = g_strdup_printf ("application/x-gst-id3v2-%c%c%c%c-frame",
+      g_ascii_tolower (frame_data[0]), g_ascii_tolower (frame_data[1]),
+      g_ascii_tolower (frame_data[2]), g_ascii_tolower (frame_data[3]));
+  caps = gst_caps_new_simple (media_type, "version", G_TYPE_INT,
+      (gint) ID3V2_VER_MAJOR (work->hdr.version), NULL);
+  gst_buffer_set_caps (blob, caps);
+  gst_caps_unref (caps);
+  g_free (media_type);
+
+  /* gst_util_dump_mem (GST_BUFFER_DATA (blob), GST_BUFFER_SIZE (blob)); */
+
+  gst_tag_list_add (work->tags, GST_TAG_MERGE_APPEND,
+      GST_ID3_DEMUX_TAG_ID3V2_FRAME, blob, NULL);
+  gst_buffer_unref (blob);
+}
+
 static ID3TagsResult
 id3demux_id3v2_frames_to_tag_list (ID3TagsWorking * work, guint size)
 {
   guint frame_hdr_size;
-  gboolean read_a_frame = FALSE;
   guint8 *start;
 
   /* Extended header if present */
@@ -423,7 +467,7 @@ id3demux_id3v2_frames_to_tag_list (ID3TagsWorking * work, guint size)
 
 #if 1
     GST_LOG
-        ("Frame @ %d (0x%02x) id %s size %d, next=%d (0x%02x) obsolete=%d\n",
+        ("Frame @ %d (0x%02x) id %s size %d, next=%d (0x%02x) obsolete=%d",
         work->hdr.frame_data - start, work->hdr.frame_data - start, frame_id,
         frame_size, work->hdr.frame_data + frame_hdr_size + frame_size - start,
         work->hdr.frame_data + frame_hdr_size + frame_size - start,
@@ -438,20 +482,35 @@ id3demux_id3v2_frames_to_tag_list (ID3TagsWorking * work, guint size)
       work->frame_flags = frame_flags;
 
       if (id3demux_id3v2_parse_frame (work)) {
-        read_a_frame = TRUE;
         GST_LOG ("Extracted frame with id %s", frame_id);
+      } else {
+        GST_LOG ("Failed to extract frame with id %s", frame_id);
+        id3demux_add_id3v2_frame_blob_to_taglist (work, frame_size);
       }
     }
     work->hdr.frame_data += frame_size;
     work->hdr.frame_data_size -= frame_size;
   }
 
-  if (!read_a_frame) {
-    GST_DEBUG ("Could not extract any frames from tag. Broken tag");
+  if (gst_structure_n_fields (GST_STRUCTURE (work->tags)) == 0) {
+    GST_DEBUG ("Could not extract any frames from tag. Broken or empty tag");
     gst_tag_list_free (work->tags);
     work->tags = NULL;
     return ID3TAGS_BROKEN_TAG;
   }
 
+  /* Set day/month now if they were in a separate (obsolete) TDAT frame */
+  if (work->pending_day != 0 && work->pending_month != 0) {
+    GDate *date = NULL;
+
+    if (gst_tag_list_get_date (work->tags, GST_TAG_DATE, &date)) {
+      g_date_set_day (date, work->pending_day);
+      g_date_set_month (date, work->pending_month);
+      gst_tag_list_add (work->tags, GST_TAG_MERGE_REPLACE, GST_TAG_DATE,
+          date, NULL);
+      g_date_free (date);
+    }
+  }
+
   return ID3TAGS_READ_TAG;
 }
