@@ -71,6 +71,7 @@
 #define GST_MATROSKA_ID_TRACKMINCACHE 0x6DE7
 #define GST_MATROSKA_ID_TRACKMAXCACHE 0x6DF8
 #define GST_MATROSKA_ID_TRACKDEFAULTDURATION 0x23E383
+#define GST_MATROSKA_ID_CONTENTENCODINGS 0x6D80
 
 /* IDs in the trackvideo master */
 #define GST_MATROSKA_ID_VIDEOFRAMERATE 0x2383E3
@@ -126,6 +127,21 @@
 #define GST_MATROSKA_ID_BLOCK      0xA1
 #define GST_MATROSKA_ID_BLOCKDURATION 0x9B
 
+/* IDs in the contentencodings master */
+#define GST_MATROSKA_ID_CONTENTENCODING 0x6240
+
+/* IDS IN THE CONTENTENCODING MASTER */
+#define GST_MATROSKA_ID_CONTENTENCODINGORDER 0X5031
+#define GST_MATROSKA_ID_CONTENTENCODINGSCOPE 0X5032
+#define GST_MATROSKA_ID_CONTENTENCODINGTYPE  0X5033
+#define GST_MATROSKA_ID_CONTENTCOMPRESSION   0X5034
+#define GST_MATROSKA_ID_CONTENTENCRYPTION    0X5035
+
+/* IDS IN THE CONTENTCOMPRESSION MASTER */
+#define GST_MATROSKA_ID_CONTENTCOMPALGO      0X4254
+#define GST_MATROSKA_ID_CONTENTCOMPSETTINGS  0X4255
+
+
 /*
  * Matroska Codec IDs. Strings.
  */
@@ -144,6 +160,7 @@
 #define GST_MATROSKA_CODEC_ID_VIDEO_REALVIDEO2   "V_REAL/RV20"
 #define GST_MATROSKA_CODEC_ID_VIDEO_REALVIDEO3   "V_REAL/RV30"
 #define GST_MATROSKA_CODEC_ID_VIDEO_REALVIDEO4   "V_REAL/RV40"
+#define GST_MATROSKA_CODEC_ID_VIDEO_THEORA       "V_THEORA"
 /* TODO: Quicktime */
 
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L1     "A_MPEG/L1"
@@ -155,6 +172,7 @@
 #define GST_MATROSKA_CODEC_ID_AUDIO_AC3          "A_AC3"
 #define GST_MATROSKA_CODEC_ID_AUDIO_DTS          "A_DTS"
 #define GST_MATROSKA_CODEC_ID_AUDIO_VORBIS       "A_VORBIS"
+#define GST_MATROSKA_CODEC_ID_AUDIO_FLAC         "A_FLAC"
 #define GST_MATROSKA_CODEC_ID_AUDIO_ACM          "A_MS/ACM"
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG2        "A_AAC/MPEG2/"
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG4        "A_AAC/MPEG4/"
@@ -173,6 +191,7 @@
 #define GST_MATROSKA_CODEC_ID_SUBTITLE_SSA       "S_TEXT/SSA"
 #define GST_MATROSKA_CODEC_ID_SUBTITLE_ASS       "S_TEXT/ASS" 
 #define GST_MATROSKA_CODEC_ID_SUBTITLE_USF       "S_TEXT/USF"
+#define GST_MATROSKA_CODEC_ID_SUBTITLE_VOBSUB    "S_VOBSUB"
 
 /*
  * Matrodka tags. Strings.
@@ -247,13 +266,35 @@ typedef struct _GstMatroskaTrackContext {
   guint64       pos;
 
   gboolean      set_discont; /* TRUE = set DISCONT flag on next buffer */
+
+  /* Special flag for Vorbis and Theora, for which we need to send
+   * codec_priv first before sending any data, and just testing
+   * for time == 0 is not enough to detect that. Used by demuxer */
+  gboolean      send_xiph_headers;
+
+  /* Special flag for Flac, for which we need to reconstruct the header
+   * buffer from the codec_priv data before sending any data, and just
+   * testing for time == 0 is not enough to detect that. Used by demuxer */
+  gboolean      send_flac_headers;
+
+  /* Special counter for muxer to skip the first N vorbis/theora headers -
+   * they are put into codec private data, not muxed into the stream */
+  guint         xiph_headers_to_skip;
+
+  /* Tags to send after newsegment event */
+  GstTagList   *pending_tags;
+
+  /* A GArray of GstMatroskaTrackEncoding structures which contain the
+   * encoding (compression/encryption) settings for this track, if any */
+  GArray       *encodings;
 } GstMatroskaTrackContext;
 
 typedef struct _GstMatroskaTrackVideoContext {
   GstMatroskaTrackContext parent;
 
-  guint         pixel_width, pixel_height,
-                display_width, display_height;
+  guint         pixel_width, pixel_height;
+  guint         display_width, display_height;
+  gdouble       default_fps;
   GstMatroskaEyeMode eye_mode;
   GstMatroskaAspectRatioMode asr_mode;
   guint32       fourcc;
@@ -263,11 +304,6 @@ typedef struct _GstMatroskaTrackAudioContext {
   GstMatroskaTrackContext parent;
 
   guint         samplerate, channels, bitdepth;
-  
-  /* Special flag for Vorbis, we need to send codec_priv first before
-   * sending any data, and just testing for time == 0 is not enough
-   * to detect that */
-  gboolean      first_frame;
 } GstMatroskaTrackAudioContext;
 
 typedef struct _GstMatroskaTrackComplexContext {
@@ -279,7 +315,8 @@ typedef struct _GstMatroskaTrackComplexContext {
 typedef struct _GstMatroskaTrackSubtitleContext {
   GstMatroskaTrackContext parent;
 
-  /* or here... */
+  gboolean    check_utf8;     /* buffers should be valid UTF-8 */
+  gboolean    invalid_utf8;   /* work around broken files      */
 } GstMatroskaTrackSubtitleContext;
 
 typedef struct _GstMatroskaIndex {
@@ -301,6 +338,15 @@ typedef struct _Wavpack4Header {
   guint32 crc;           /* crc for actual decoded data                    */
 } Wavpack4Header;
 
+typedef struct _GstMatroskaTrackEncoding {
+  guint   order;
+  guint   scope     : 3;
+  guint   type      : 1;
+  guint   comp_algo : 2;
+  guint8 *comp_settings;
+  guint   comp_settings_length;
+} GstMatroskaTrackEncoding;
+
 gboolean gst_matroska_track_init_video_context    (GstMatroskaTrackContext ** p_context);
 gboolean gst_matroska_track_init_audio_context    (GstMatroskaTrackContext ** p_context);
 gboolean gst_matroska_track_init_subtitle_context (GstMatroskaTrackContext ** p_context);
