@@ -54,8 +54,9 @@ GST_DEBUG_CATEGORY_STATIC (gst_multipart_mux_debug);
 #define GST_TYPE_MULTIPART_MUX (gst_multipart_mux_get_type())
 #define GST_MULTIPART_MUX(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_MULTIPART_MUX, GstMultipartMux))
 #define GST_MULTIPART_MUX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_MULTIPART_MUX, GstMultipartMux))
+#define GST_MULTIPART_MUX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_MULTIPART_MUX, GstMultipartMuxClass))
 #define GST_IS_MULTIPART_MUX(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_MULTIPART_MUX))
-#define GST_IS_MULTIPART_MUX_CLASS(obj) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_MULTIPART_MUX))
+#define GST_IS_MULTIPART_MUX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_MULTIPART_MUX))
 
 typedef struct _GstMultipartMux GstMultipartMux;
 typedef struct _GstMultipartMuxClass GstMultipartMuxClass;
@@ -66,6 +67,8 @@ typedef struct
   GstCollectData collect;       /* we extend the CollectData */
 
   GstBuffer *buffer;            /* the queued buffer for this pad */
+  GstClockTime timestamp;       /* its timestamp, converted to running_time so that we can
+                                   correctly sort over multiple segments. */
 }
 GstMultipartPad;
 
@@ -93,11 +96,14 @@ struct _GstMultipartMux
   gchar *boundary;
 
   gboolean negotiated;
+  gboolean need_segment;
 };
 
 struct _GstMultipartMuxClass
 {
   GstElementClass parent_class;
+
+  GHashTable *mimetypes;
 };
 
 /* elementfactory information */
@@ -128,6 +134,18 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%d",
     GST_STATIC_CAPS_ANY         /* we can take anything, really */
     );
 
+typedef struct
+{
+  const gchar *key;
+  const gchar *val;
+} MimeTypeMap;
+
+/* convert from gst structure names to mime types. Add more when needed. */
+static const MimeTypeMap mimetypes[] = {
+  {"audio/x-mulaw", "audio/basic"},
+  {NULL, NULL}
+};
+
 static void gst_multipart_mux_base_init (gpointer g_class);
 static void gst_multipart_mux_class_init (GstMultipartMuxClass * klass);
 static void gst_multipart_mux_init (GstMultipartMux * multipart_mux);
@@ -194,6 +212,7 @@ gst_multipart_mux_class_init (GstMultipartMuxClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
+  gint i;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
@@ -210,6 +229,13 @@ gst_multipart_mux_class_init (GstMultipartMuxClass * klass)
 
   gstelement_class->request_new_pad = gst_multipart_mux_request_new_pad;
   gstelement_class->change_state = gst_multipart_mux_change_state;
+
+  /* populate mime types */
+  klass->mimetypes = g_hash_table_new (g_str_hash, g_str_equal);
+  for (i = 0; mimetypes[i].key; i++) {
+    g_hash_table_insert (klass->mimetypes, (gpointer) mimetypes[i].key,
+        (gpointer) mimetypes[i].val);
+  }
 }
 
 static void
@@ -225,7 +251,6 @@ gst_multipart_mux_init (GstMultipartMux * multipart_mux)
   gst_element_add_pad (GST_ELEMENT (multipart_mux), multipart_mux->srcpad);
 
   multipart_mux->boundary = g_strdup (DEFAULT_BOUNDARY);
-  multipart_mux->negotiated = FALSE;
 
   multipart_mux->collect = gst_collect_pads_new ();
   gst_collect_pads_set_function (multipart_mux->collect,
@@ -240,36 +265,14 @@ gst_multipart_mux_finalize (GObject * object)
 
   multipart_mux = GST_MULTIPART_MUX (object);
 
-  if (multipart_mux->collect) {
+  g_free (multipart_mux->boundary);
+
+  if (multipart_mux->collect)
     gst_object_unref (multipart_mux->collect);
-    multipart_mux->collect = NULL;
-  }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static GstPadLinkReturn
-gst_multipart_mux_sinkconnect (GstPad * pad, GstPad * peer)
-{
-  GstMultipartMux *multipart_mux;
-  GstMultipartPad *mppad;
-  gchar *pad_name = NULL;
-
-  multipart_mux = GST_MULTIPART_MUX (gst_pad_get_parent (pad));
-
-  mppad = (GstMultipartPad *) gst_pad_get_element_private (pad);
-
-  pad_name = gst_pad_get_name (pad);
-
-  GST_DEBUG_OBJECT (multipart_mux, "sinkconnect triggered on %s", pad_name);
-
-  g_free (pad_name);
-
-  gst_object_unref (multipart_mux);
-
-  return GST_PAD_LINK_OK;
-}
-
 static GstPad *
 gst_multipart_mux_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * req_name)
@@ -277,51 +280,43 @@ gst_multipart_mux_request_new_pad (GstElement * element,
   GstMultipartMux *multipart_mux;
   GstPad *newpad;
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (element);
+  gchar *name;
 
-  g_return_val_if_fail (templ != NULL, NULL);
-
-  if (templ->direction != GST_PAD_SINK) {
-    g_warning ("multipart_mux: request pad that is not a SINK pad\n");
-    return NULL;
-  }
-
-  g_return_val_if_fail (GST_IS_MULTIPART_MUX (element), NULL);
+  if (templ != gst_element_class_get_pad_template (klass, "sink_%d"))
+    goto wrong_template;
 
   multipart_mux = GST_MULTIPART_MUX (element);
 
-  if (templ == gst_element_class_get_pad_template (klass, "sink_%d")) {
-    gchar *name;
+  /* create new pad with the name */
+  name = g_strdup_printf ("sink_%02d", multipart_mux->numpads);
+  newpad = gst_pad_new_from_template (templ, name);
+  g_free (name);
 
-    /* create new pad with the name */
-    name = g_strdup_printf ("sink_%02d", multipart_mux->numpads);
-    newpad = gst_pad_new_from_template (templ, name);
-    g_free (name);
+  /* construct our own wrapper data structure for the pad to
+   * keep track of its status */
+  {
+    GstMultipartPad *multipartpad;
 
-    /* construct our own wrapper data structure for the pad to
-     * keep track of its status */
-    {
-      GstMultipartPad *multipartpad;
+    multipartpad = (GstMultipartPad *)
+        gst_collect_pads_add_pad (multipart_mux->collect, newpad,
+        sizeof (GstMultipartPad));
 
-      multipartpad = (GstMultipartPad *)
-          gst_collect_pads_add_pad (multipart_mux->collect, newpad,
-          sizeof (GstMultipartPad));
-
-      /* save a pointer to our data in the pad */
-      gst_pad_set_element_private (newpad, multipartpad);
-      multipart_mux->numpads++;
-    }
-  } else {
-    g_warning ("multipart_mux: this is not our template!\n");
-    return NULL;
+    /* save a pointer to our data in the pad */
+    gst_pad_set_element_private (newpad, multipartpad);
+    multipart_mux->numpads++;
   }
 
-  /* setup some pad functions */
-  gst_pad_set_link_function (newpad, gst_multipart_mux_sinkconnect);
-
-  /* dd the pad to the element */
+  /* add the pad to the element */
   gst_element_add_pad (element, newpad);
 
   return newpad;
+
+  /* ERRORS */
+wrong_template:
+  {
+    g_warning ("multipart_mux: this is not our template!");
+    return NULL;
+  }
 }
 
 /* handle events */
@@ -348,6 +343,26 @@ gst_multipart_mux_handle_src_event (GstPad * pad, GstEvent * event)
   return gst_pad_event_default (pad, event);
 }
 
+static const gchar *
+gst_multipart_mux_get_mime (GstMultipartMux * mux, GstStructure * s)
+{
+  GstMultipartMuxClass *klass;
+  const gchar *mime;
+  const gchar *name;
+
+  klass = GST_MULTIPART_MUX_GET_CLASS (mux);
+
+  name = gst_structure_get_name (s);
+
+  /* use hashtable to convert to mime type */
+  mime = g_hash_table_lookup (klass->mimetypes, name);
+  if (mime == NULL) {
+    /* no mime type mapping, use name */
+    mime = name;
+  }
+  return mime;
+}
+
 /*
  * Given two pads, compare the buffers queued on it and return 0 if they have
  * an equal priority, 1 if the new pad is better, -1 if the old pad is better 
@@ -366,12 +381,12 @@ gst_multipart_mux_compare_pads (GstMultipartMux * multipart_mux,
     return -1;
 
   /* no timestamp on old buffer, it must go first */
-  oldtime = GST_BUFFER_TIMESTAMP (old->buffer);
+  oldtime = old->timestamp;
   if (oldtime == GST_CLOCK_TIME_NONE)
     return -1;
 
   /* no timestamp on new buffer, it must go first */
-  newtime = GST_BUFFER_TIMESTAMP (new->buffer);
+  newtime = new->timestamp;
   if (newtime == GST_CLOCK_TIME_NONE)
     return 1;
 
@@ -410,9 +425,13 @@ gst_multipart_mux_queue_pads (GstMultipartMux * mux)
 
       buf = gst_collect_pads_pop (mux->collect, data);
 
-      /* Adjust timestamp with segment_start and preroll */
-      if (buf) {
-        GST_BUFFER_TIMESTAMP (buf) -= data->segment.start;
+      /* Store timestamp with segment_start and preroll */
+      if (buf && GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+        pad->timestamp =
+            gst_segment_to_running_time (&data->segment, GST_FORMAT_TIME,
+            GST_BUFFER_TIMESTAMP (buf));
+      } else {
+        pad->timestamp = GST_CLOCK_TIME_NONE;
       }
 
       pad->buffer = buf;
@@ -434,7 +453,8 @@ gst_multipart_mux_queue_pads (GstMultipartMux * mux)
  *
  * 1) find a pad to pull on, this is done by pulling on all pads and
  *    looking at the buffers to decide which one should be muxed first.
- * 2) push buffer on best pad, go to 1
+ * 2) create a new buffer for the header
+ * 3) push both buffers on best pad, go to 1
  */
 static GstFlowReturn
 gst_multipart_mux_collected (GstCollectPads * pads, GstMultipartMux * mux)
@@ -442,24 +462,21 @@ gst_multipart_mux_collected (GstCollectPads * pads, GstMultipartMux * mux)
   GstMultipartPad *best;
   GstFlowReturn ret = GST_FLOW_OK;
   gchar *header = NULL;
-  size_t newlen, headerlen;
-  GstBuffer *newbuf = NULL;
+  size_t headerlen;
+  GstBuffer *headerbuf = NULL;
+  GstBuffer *databuf = NULL;
   GstStructure *structure = NULL;
+  const gchar *mime;
 
   GST_DEBUG_OBJECT (mux, "all pads are collected");
 
   /* queue buffers on all pads; find a buffer with the lowest timestamp */
   best = gst_multipart_mux_queue_pads (mux);
-  if (best && !best->buffer)
-    goto beach;
-
-  /* EOS */
-  if (!best) {
-    GST_DEBUG_OBJECT (mux, "Pushing EOS");
-    gst_pad_push_event (mux->srcpad, gst_event_new_eos ());
-    ret = GST_FLOW_WRONG_STATE;
-    goto beach;
-  }
+  if (!best)
+    /* EOS */
+    goto eos;
+  else if (!best->buffer)
+    goto buffer_error;
 
   /* If not negotiated yet set caps on src pad */
   if (!mux->negotiated) {
@@ -468,54 +485,127 @@ gst_multipart_mux_collected (GstCollectPads * pads, GstMultipartMux * mux)
     newcaps = gst_caps_new_simple ("multipart/x-mixed-replace",
         "boundary", G_TYPE_STRING, mux->boundary, NULL);
 
-    if (gst_pad_set_caps (mux->srcpad, newcaps)) {
-      mux->negotiated = TRUE;
-    } else {
-      GST_ELEMENT_ERROR (mux, CORE, NEGOTIATION, (NULL), (NULL));
-      ret = GST_FLOW_UNEXPECTED;
-      goto beach;
-    }
+    if (!gst_pad_set_caps (mux->srcpad, newcaps))
+      goto nego_error;
+
+    mux->negotiated = TRUE;
+  }
+
+  /* see if we need to push a segment */
+  if (mux->need_segment) {
+    GstEvent *event;
+    GstClockTime time;
+
+    if (best->timestamp != -1)
+      time = best->timestamp;
+    else
+      time = 0;
+
+    /* for the segment, we take the first timestamp we see, we don't know the
+     * length and the position is 0 */
+    event = gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME,
+        time, -1, 0);
+
+    gst_pad_push_event (mux->srcpad, event);
+
+    mux->need_segment = FALSE;
   }
 
   structure = gst_caps_get_structure (GST_BUFFER_CAPS (best->buffer), 0);
-  if (!structure) {
-    GST_WARNING_OBJECT (mux, "no caps on the incoming buffer %p", best->buffer);
+  if (!structure)
+    goto no_caps;
+
+  /* get the mime type for the structure */
+  mime = gst_multipart_mux_get_mime (mux, structure);
+
+  header = g_strdup_printf ("\r\n--%s\r\nContent-Type: %s\r\n"
+      "Content-Length: %u\r\n\r\n",
+      mux->boundary, mime, GST_BUFFER_SIZE (best->buffer));
+  headerlen = strlen (header);
+
+  ret = gst_pad_alloc_buffer_and_set_caps (mux->srcpad, GST_BUFFER_OFFSET_NONE,
+      headerlen, GST_PAD_CAPS (mux->srcpad), &headerbuf);
+  if (ret != GST_FLOW_OK)
+    goto alloc_failed;
+
+  memcpy (GST_BUFFER_DATA (headerbuf), header, headerlen);
+  g_free (header);
+
+  /* the header has the same timestamp as the data buffer (which we will push
+   * below) and has a duration of 0 */
+  GST_BUFFER_TIMESTAMP (headerbuf) = best->timestamp;
+  GST_BUFFER_DURATION (headerbuf) = 0;
+  GST_BUFFER_OFFSET (headerbuf) = mux->offset;
+  mux->offset += headerlen;
+  GST_BUFFER_OFFSET_END (headerbuf) = mux->offset;
+
+  GST_DEBUG_OBJECT (mux, "pushing %" G_GSIZE_FORMAT " bytes header buffer",
+      headerlen);
+  ret = gst_pad_push (mux->srcpad, headerbuf);
+  if (ret != GST_FLOW_OK)
+    /* push always takes ownership of the buffer, even after an error, so we
+     * don't need to unref headerbuf here. */
+    goto beach;
+
+  /* take best->buffer, we don't need to unref it later as we will push it
+   * now. */
+  databuf = gst_buffer_make_metadata_writable (best->buffer);
+  best->buffer = NULL;
+
+  gst_buffer_set_caps (databuf, GST_PAD_CAPS (mux->srcpad));
+  /* we need to updated the timestamp to match the running_time */
+  GST_BUFFER_TIMESTAMP (databuf) = best->timestamp;
+  GST_BUFFER_OFFSET (databuf) = mux->offset;
+  mux->offset += GST_BUFFER_SIZE (databuf);
+  GST_BUFFER_OFFSET_END (databuf) = mux->offset;
+
+  GST_DEBUG_OBJECT (mux, "pushing %u bytes data buffer",
+      GST_BUFFER_SIZE (databuf));
+  ret = gst_pad_push (mux->srcpad, databuf);
+
+beach:
+  if (best && best->buffer) {
+    gst_buffer_unref (best->buffer);
+    best->buffer = NULL;
+  }
+  return ret;
+
+  /* ERRORS */
+buffer_error:
+  {
+    /* There is a best but no buffer, this is not quite right.. */
+    GST_ELEMENT_ERROR (mux, STREAM, FAILED, (NULL), ("internal muxing error"));
+    ret = GST_FLOW_ERROR;
     goto beach;
   }
-
-  header = g_strdup_printf ("\n--%s\nContent-type: %s\n\n",
-      mux->boundary, gst_structure_get_name (structure));
-
-  headerlen = strlen (header);
-  newlen = headerlen + GST_BUFFER_SIZE (best->buffer);
-
-  ret =
-      gst_pad_alloc_buffer_and_set_caps (mux->srcpad, GST_BUFFER_OFFSET_NONE,
-      newlen, GST_PAD_CAPS (mux->srcpad), &newbuf);
-  if (ret != GST_FLOW_OK) {
-    GST_WARNING_OBJECT (mux, "failed allocating a %d bytes buffer", newlen);
+eos:
+  {
+    GST_DEBUG_OBJECT (mux, "Pushing EOS");
+    gst_pad_push_event (mux->srcpad, gst_event_new_eos ());
+    ret = GST_FLOW_UNEXPECTED;
+    goto beach;
+  }
+nego_error:
+  {
+    GST_WARNING_OBJECT (mux, "failed to set caps");
+    GST_ELEMENT_ERROR (mux, CORE, NEGOTIATION, (NULL), (NULL));
+    ret = GST_FLOW_NOT_NEGOTIATED;
+    goto beach;
+  }
+no_caps:
+  {
+    GST_WARNING_OBJECT (mux, "no caps on the incoming buffer %p", best->buffer);
+    GST_ELEMENT_ERROR (mux, CORE, NEGOTIATION, (NULL), (NULL));
+    ret = GST_FLOW_NOT_NEGOTIATED;
+    goto beach;
+  }
+alloc_failed:
+  {
+    GST_WARNING_OBJECT (mux,
+        "failed allocating a %" G_GSIZE_FORMAT " bytes buffer", headerlen);
     g_free (header);
     goto beach;
   }
-
-  memcpy (GST_BUFFER_DATA (newbuf), header, headerlen);
-  memcpy (GST_BUFFER_DATA (newbuf) + headerlen,
-      GST_BUFFER_DATA (best->buffer), GST_BUFFER_SIZE (best->buffer));
-
-  gst_buffer_stamp (newbuf, best->buffer);
-  GST_BUFFER_OFFSET (newbuf) = mux->offset;
-
-  g_free (header);
-
-  mux->offset += newlen;
-
-  gst_pad_push (mux->srcpad, newbuf);
-
-  gst_buffer_unref (best->buffer);
-  best->buffer = NULL;
-
-beach:
-  return ret;
 }
 
 static void
@@ -564,11 +654,10 @@ gst_multipart_mux_change_state (GstElement * element, GstStateChange transition)
   multipart_mux = GST_MULTIPART_MUX (element);
 
   switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      multipart_mux->negotiated = FALSE;
-      break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       multipart_mux->offset = 0;
+      multipart_mux->negotiated = FALSE;
+      multipart_mux->need_segment = TRUE;
       GST_DEBUG_OBJECT (multipart_mux, "starting collect pads");
       gst_collect_pads_start (multipart_mux->collect);
       break;
