@@ -9,7 +9,12 @@
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more 
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -22,12 +27,12 @@
 
 #include "gstrtpmp4gpay.h"
 
-GST_DEBUG_CATEGORY (rtpmp4gpay_debug);
+GST_DEBUG_CATEGORY_STATIC (rtpmp4gpay_debug);
 #define GST_CAT_DEFAULT (rtpmp4gpay_debug)
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_mp4gpay_details =
-GST_ELEMENT_DETAILS ("RTP packet parser",
+GST_ELEMENT_DETAILS ("RTP packet payloader",
     "Codec/Payloader/Network",
     "Payload MPEG4 elementary streams as RTP packets (RFC 3640)",
     "Wim Taymans <wim@fluendo.com>");
@@ -46,29 +51,30 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
-        "media = (string) \"video\", "
-        "payload = (int) [ 96, 127 ], "
+        "media = (string) { \"video\", \"audio\", \"application\" }, "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) [1, MAX ], "
-        "encoding-name = (string) \"mpeg4-generic\", "
+        "encoding-name = (string) \"MPEG4-GENERIC\", "
         /* required string params */
         "streamtype = (string) { \"4\", \"5\" }, "      /* 4 = video, 5 = audio */
-        "profile-level-id = (int) [1,MAX], "
+        /* "profile-level-id = (string) [1,MAX], " */
         /* "config = (string) [1,MAX]" */
-        "mode = (string) { \"generic\", \"CELP-cbr\", \"CELP-vbr\", \"AAC-lbr\", \"AAC-hbr\" }, "
+        "mode = (string) { \"generic\", \"CELP-cbr\", \"CELP-vbr\", \"AAC-lbr\", \"AAC-hbr\" } "
         /* Optional general parameters */
-        "objecttype = (int) [1,MAX], " "constantsize = (int) [1,MAX], " /* constant size of each AU */
-        "constantduration = (int) [1,MAX], "    /* constant duration of each AU */
-        "maxdisplacement = (int) [1,MAX], "
-        "de-interleavebuffersize: = (int) [1,MAX], "
+        /* "objecttype = (string) [1,MAX], " */
+        /* "constantsize = (string) [1,MAX], " *//* constant size of each AU */
+        /* "constantduration = (string) [1,MAX], " *//* constant duration of each AU */
+        /* "maxdisplacement = (string) [1,MAX], " */
+        /* "de-interleavebuffersize = (string) [1,MAX], " */
         /* Optional configuration parameters */
-        "sizelength = (int) [1, 16], "  /* max 16 bits, should be enough... */
-        "indexlength = (int) [1, 8], "
-        "indexdeltalength = (int) [1, 8], "
-        "ctsdeltalength = (int) [1, 64], "
-        "dtsdeltalength = (int) [1, 64], "
-        "randomaccessindication = (int) {0, 1}, "
-        "streamstateindication = (int) [0, 64], "
-        "auxiliarydatasizelength = (int) [0, 64]")
+        /* "sizelength = (string) [1, 16], " *//* max 16 bits, should be enough... */
+        /* "indexlength = (string) [1, 8], " */
+        /* "indexdeltalength = (string) [1, 8], " */
+        /* "ctsdeltalength = (string) [1, 64], " */
+        /* "dtsdeltalength = (string) [1, 64], " */
+        /* "randomaccessindication = (string) {0, 1}, " */
+        /* "streamstateindication = (string) [0, 64], " */
+        /* "auxiliarydatasizelength = (string) [0, 64]" */ )
     );
 
 enum
@@ -155,7 +161,6 @@ gst_rtp_mp4g_pay_class_init (GstRtpMP4GPayClass * klass)
 
   GST_DEBUG_CATEGORY_INIT (rtpmp4gpay_debug, "rtpmp4gpay", 0,
       "MP4-generic RTP Payloader");
-
 }
 
 static void
@@ -163,7 +168,7 @@ gst_rtp_mp4g_pay_init (GstRtpMP4GPay * rtpmp4gpay)
 {
   rtpmp4gpay->adapter = gst_adapter_new ();
   rtpmp4gpay->rate = 90000;
-  rtpmp4gpay->profile = 1;
+  rtpmp4gpay->profile = g_strdup ("1");
   rtpmp4gpay->mode = "";
 }
 
@@ -176,6 +181,11 @@ gst_rtp_mp4g_pay_finalize (GObject * object)
 
   g_object_unref (rtpmp4gpay->adapter);
   rtpmp4gpay->adapter = NULL;
+  g_free (rtpmp4gpay->params);
+  rtpmp4gpay->params = NULL;
+
+  g_free (rtpmp4gpay->profile);
+  rtpmp4gpay->profile = NULL;
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -201,10 +211,10 @@ gst_rtp_mp4g_pay_parse_audio_config (GstRtpMP4GPay * rtpmp4gpay,
   if (size < 2)
     goto too_short;
 
-  /* only AAC LC for now */
+  /* any object type is fine, we need to copy it to the profile-level-id field. */
   objectType = (data[0] & 0xf8) >> 3;
-  if (objectType != 2)
-    goto unsupported_type;
+  if (objectType == 0)
+    goto invalid_object;
 
   samplingIdx = ((data[0] & 0x07) << 1) | ((data[1] & 0x80) >> 7);
   /* only fixed values for now */
@@ -228,13 +238,15 @@ gst_rtp_mp4g_pay_parse_audio_config (GstRtpMP4GPay * rtpmp4gpay,
     rtpmp4gpay->rate = sampling_table[samplingIdx];
   }
   /* extra rtp params contain the number of channels */
-  rtpmp4gpay->params = channelCfg;
+  g_free (rtpmp4gpay->params);
+  rtpmp4gpay->params = g_strdup_printf ("%d", channelCfg);
   /* audio stream type */
-  rtpmp4gpay->streamtype = 5;
-  /* mode */
-  rtpmp4gpay->mode = "ACC-hbr";
-  /* profile (should be 1) */
-  rtpmp4gpay->profile = objectType - 1;
+  rtpmp4gpay->streamtype = "5";
+  /* mode only high bitrate for now */
+  rtpmp4gpay->mode = "AAC-hbr";
+  /* profile */
+  g_free (rtpmp4gpay->profile);
+  rtpmp4gpay->profile = g_strdup_printf ("%d", objectType);
 
   GST_DEBUG_OBJECT (rtpmp4gpay,
       "objectType: %d, samplingIdx: %d (%d), channelCfg: %d", objectType,
@@ -246,13 +258,13 @@ gst_rtp_mp4g_pay_parse_audio_config (GstRtpMP4GPay * rtpmp4gpay,
 too_short:
   {
     GST_ELEMENT_ERROR (rtpmp4gpay, STREAM, FORMAT,
-        (NULL), ("config string too short"));
+        (NULL), ("config string too short, expected 2 bytes, got %d", size));
     return FALSE;
   }
-unsupported_type:
+invalid_object:
   {
-    GST_ELEMENT_ERROR (rtpmp4gpay, STREAM, NOT_IMPLEMENTED,
-        (NULL), ("unsupported object type %d", objectType));
+    GST_ELEMENT_ERROR (rtpmp4gpay, STREAM, FORMAT,
+        (NULL), ("invalid object type 0"));
     return FALSE;
   }
 wrong_freq:
@@ -264,7 +276,7 @@ wrong_freq:
 wrong_channels:
   {
     GST_ELEMENT_ERROR (rtpmp4gpay, STREAM, NOT_IMPLEMENTED,
-        (NULL), ("unsupported number of channels %d", channelCfg));
+        (NULL), ("unsupported number of channels %d, must < 8", channelCfg));
     return FALSE;
   }
 }
@@ -286,25 +298,27 @@ gst_rtp_mp4g_pay_parse_video_config (GstRtpMP4GPay * rtpmp4gpay,
     goto too_short;
 
   code = GST_READ_UINT32_BE (data);
+
+  g_free (rtpmp4gpay->profile);
   if (code == VOS_STARTCODE) {
     /* get profile */
-    rtpmp4gpay->profile = data[4];
+    rtpmp4gpay->profile = g_strdup_printf ("%d", (gint) data[4]);
   } else {
     GST_ELEMENT_WARNING (rtpmp4gpay, STREAM, FORMAT,
-        (NULL), ("profile not found in config string"));
-    rtpmp4gpay->profile = 1;
+        (NULL), ("profile not found in config string, assuming \'1\'"));
+    rtpmp4gpay->profile = g_strdup ("1");
   }
 
   /* fixed rate */
   rtpmp4gpay->rate = 90000;
   /* video stream type */
-  rtpmp4gpay->streamtype = 4;
+  rtpmp4gpay->streamtype = "4";
   /* no params for video */
-  rtpmp4gpay->params = 0;
+  rtpmp4gpay->params = NULL;
   /* mode */
   rtpmp4gpay->mode = "generic";
 
-  GST_LOG_OBJECT (rtpmp4gpay, "profile %d", rtpmp4gpay->profile);
+  GST_LOG_OBJECT (rtpmp4gpay, "profile %s", rtpmp4gpay->profile);
 
   return TRUE;
 
@@ -323,14 +337,14 @@ gst_rtp_mp4g_pay_new_caps (GstRtpMP4GPay * rtpmp4gpay)
   gchar *config;
   GValue v = { 0 };
 
-#define MP4GCAPS					\
-  "streamtype", G_TYPE_INT, rtpmp4gpay->streamtype, 	\
-  "profile-level-id", G_TYPE_INT, rtpmp4gpay->profile,	\
-  "mode", G_TYPE_STRING, rtpmp4gpay->mode,		\
-  "config", G_TYPE_STRING, config,			\
-  "sizelength", G_TYPE_INT, 13,				\
-  "indexlength", G_TYPE_INT, 3,				\
-  "indexdeltalength", G_TYPE_INT, 3,			\
+#define MP4GCAPS						\
+  "streamtype", G_TYPE_STRING, rtpmp4gpay->streamtype, 		\
+  "profile-level-id", G_TYPE_STRING, rtpmp4gpay->profile,	\
+  "mode", G_TYPE_STRING, rtpmp4gpay->mode,			\
+  "config", G_TYPE_STRING, config,				\
+  "sizelength", G_TYPE_STRING, "13",				\
+  "indexlength", G_TYPE_STRING, "3",				\
+  "indexdeltalength", G_TYPE_STRING, "3",			\
   NULL
 
   g_value_init (&v, GST_TYPE_BUFFER);
@@ -340,7 +354,7 @@ gst_rtp_mp4g_pay_new_caps (GstRtpMP4GPay * rtpmp4gpay)
   /* hmm, silly */
   if (rtpmp4gpay->params) {
     gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4gpay),
-        "encoding-params", G_TYPE_INT, rtpmp4gpay->params, MP4GCAPS);
+        "encoding-params", G_TYPE_STRING, rtpmp4gpay->params, MP4GCAPS);
   } else {
     gst_basertppayload_set_outcaps (GST_BASE_RTP_PAYLOAD (rtpmp4gpay),
         MP4GCAPS);
@@ -358,7 +372,7 @@ gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
   GstRtpMP4GPay *rtpmp4gpay;
   GstStructure *structure;
   const GValue *codec_data;
-  gboolean res = TRUE;
+  gchar *media_type = NULL;
 
   rtpmp4gpay = GST_RTP_MP4G_PAY (payload);
 
@@ -370,6 +384,7 @@ gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
     if (G_VALUE_TYPE (codec_data) == GST_TYPE_BUFFER) {
       GstBuffer *buffer;
       const gchar *name;
+      gboolean res;
 
       buffer = gst_value_get_buffer (codec_data);
       GST_LOG_OBJECT (rtpmp4gpay, "configuring codec_data");
@@ -379,8 +394,10 @@ gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
       /* parse buffer */
       if (!strcmp (name, "audio/mpeg")) {
         res = gst_rtp_mp4g_pay_parse_audio_config (rtpmp4gpay, buffer);
+        media_type = "audio";
       } else if (!strcmp (name, "video/mpeg")) {
         res = gst_rtp_mp4g_pay_parse_video_config (rtpmp4gpay, buffer);
+        media_type = "video";
       } else {
         res = FALSE;
       }
@@ -394,13 +411,15 @@ gst_rtp_mp4g_pay_setcaps (GstBaseRTPPayload * payload, GstCaps * caps)
       rtpmp4gpay->config = gst_buffer_copy (buffer);
     }
   }
+  if (media_type == NULL)
+    goto config_failed;
 
-  gst_basertppayload_set_options (payload, "video", TRUE, "mpeg4-generic",
+  gst_basertppayload_set_options (payload, media_type, TRUE, "MPEG4-GENERIC",
       rtpmp4gpay->rate);
 
   gst_rtp_mp4g_pay_new_caps (rtpmp4gpay);
 
-  return res;
+  return TRUE;
 
   /* ERRORS */
 config_failed:
@@ -434,7 +453,6 @@ gst_rtp_mp4g_pay_flush (GstRtpMP4GPay * rtpmp4gpay)
   while (avail > 0) {
     guint towrite;
     guint8 *payload;
-    guint8 *data;
     guint payload_len;
     guint packet_len;
 
@@ -492,13 +510,12 @@ gst_rtp_mp4g_pay_flush (GstRtpMP4GPay * rtpmp4gpay)
     payload[2] = (total & 0x1fe0) >> 5;
     payload[3] = (total & 0x1f) << 3;   /* we use 13 bits for the size, 3 bits index */
 
-    data = (guint8 *) gst_adapter_peek (rtpmp4gpay->adapter, payload_len);
-    memcpy (&payload[4], data, payload_len);
-
+    /* copy stuff from adapter to payload */
+    gst_adapter_copy (rtpmp4gpay->adapter, &payload[4], 0, payload_len);
     gst_adapter_flush (rtpmp4gpay->adapter, payload_len);
 
     /* marker only if the packet is complete */
-    gst_rtp_buffer_set_marker (outbuf, avail > payload_len);
+    gst_rtp_buffer_set_marker (outbuf, avail <= payload_len);
 
     GST_BUFFER_TIMESTAMP (outbuf) = rtpmp4gpay->first_ts;
 
