@@ -59,7 +59,9 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"X-SV3V-ES\"")
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
+        "clock-rate = (int) 90000, "
+        "encoding-name = (string) { \"X-SV3V-ES\", \"X-SORENSON-VIDEO\" }")
     );
 
 GST_BOILERPLATE (GstRtpSV3VDepay, gst_rtp_sv3v_depay, GstBaseRTPDepayload,
@@ -250,20 +252,11 @@ gst_rtp_sv3v_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     if (M) {
       /* frame is completed: push contents of adapter */
       guint avail;
-      guint8 *data;
-      guint32 timestamp;
 
       avail = gst_adapter_available (rtpsv3vdepay->adapter);
-      data = gst_adapter_take (rtpsv3vdepay->adapter, avail);
-
-      /* create buffer with data */
-      outbuf = gst_rtp_buffer_new_take_data (data, avail);
+      outbuf = gst_adapter_take_buffer (rtpsv3vdepay->adapter, avail);
 
       /* timestamp for complete buffer is that of last buffer as well */
-      timestamp = gst_rtp_buffer_get_timestamp (buf);
-      GST_BUFFER_TIMESTAMP (outbuf) =
-          gst_util_uint64_scale_int (timestamp, GST_SECOND,
-          depayload->clock_rate);
       gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
 
       return outbuf;
@@ -344,5 +337,5 @@ gboolean
 gst_rtp_sv3v_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpsv3vdepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_SV3V_DEPAY);
+      GST_RANK_NONE, GST_TYPE_RTP_SV3V_DEPAY);
 }
