@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Wim Taymans <wim.taymans@gmail.com>
+ * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,6 +37,13 @@ GST_DEBUG_CATEGORY_STATIC (rtptheorapay_debug);
  * http://svn.xiph.org/trunk/theora/doc/draft-ietf-avt-rtp-theora-01.txt
  */
 
+/* elementfactory information */
+static const GstElementDetails gst_rtp_theorapay_details =
+GST_ELEMENT_DETAILS ("RTP packet depayloader",
+    "Codec/Payloader/Network",
+    "Payload-encode Theora video into RTP packets (draft-01 RFC XXXX)",
+    "Wim Taymans <wim@fluendo.com>");
+
 static GstStaticPadTemplate gst_rtp_theora_pay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -44,18 +51,18 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
         "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
-        "clock-rate = (int) 90000, " "encoding-name = (string) \"THEORA\""
+        "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"THEORA\""
         /* All required parameters
          *
          * "sampling = (string) { "YCbCr-4:2:0", "YCbCr-4:2:2", "YCbCr-4:4:4" } "
          * "width = (string) [1, 1048561] (multiples of 16) "
          * "height = (string) [1, 1048561] (multiples of 16) "
+         * "delivery-method = (string) { inline, in_band, out_band/<specific_name> } "
          * "configuration = (string) ANY"
          */
         /* All optional parameters
          *
          * "configuration-uri ="
-         * "delivery-method = (string) { inline, in_band, out_band/<specific_name> } "
          */
     )
     );
@@ -67,31 +74,13 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("video/x-theora")
     );
 
-#define DEFAULT_CONFIG_INTERVAL 0
-
-enum
-{
-  PROP_0,
-  PROP_CONFIG_INTERVAL
-};
-
 GST_BOILERPLATE (GstRtpTheoraPay, gst_rtp_theora_pay, GstBaseRTPPayload,
     GST_TYPE_BASE_RTP_PAYLOAD);
 
 static gboolean gst_rtp_theora_pay_setcaps (GstBaseRTPPayload * basepayload,
     GstCaps * caps);
-static GstStateChangeReturn gst_rtp_theora_pay_change_state (GstElement *
-    element, GstStateChange transition);
 static GstFlowReturn gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * pad,
     GstBuffer * buffer);
-static gboolean gst_rtp_theora_pay_handle_event (GstPad * pad,
-    GstEvent * event);
-
-
-static void gst_rtp_theora_pay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_theora_pay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
 
 static void
 gst_rtp_theora_pay_base_init (gpointer klass)
@@ -103,10 +92,7 @@ gst_rtp_theora_pay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_theora_pay_sink_template));
 
-  gst_element_class_set_details_simple (element_class, "RTP Theora payloader",
-      "Codec/Payloader/Network/RTP",
-      "Payload-encode Theora video into RTP packets (draft-01 RFC XXXX)",
-      "Wim Taymans <wim.taymans@gmail.com>");
+  gst_element_class_set_details (element_class, &gst_rtp_theorapay_details);
 }
 
 static void
@@ -120,55 +106,19 @@ gst_rtp_theora_pay_class_init (GstRtpTheoraPayClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
-  gstelement_class->change_state = gst_rtp_theora_pay_change_state;
+  parent_class = g_type_class_peek_parent (klass);
 
   gstbasertppayload_class->set_caps = gst_rtp_theora_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_theora_pay_handle_buffer;
-  gstbasertppayload_class->handle_event = gst_rtp_theora_pay_handle_event;
-
-  gobject_class->set_property = gst_rtp_theora_pay_set_property;
-  gobject_class->get_property = gst_rtp_theora_pay_get_property;
 
   GST_DEBUG_CATEGORY_INIT (rtptheorapay_debug, "rtptheorapay", 0,
       "Theora RTP Payloader");
-
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CONFIG_INTERVAL,
-      g_param_spec_uint ("config-interval", "Config Send Interval",
-          "Send Config Insertion Interval in seconds (configuration headers "
-          "will be multiplexed in the data stream when detected.) (0 = disabled)",
-          0, 3600, DEFAULT_CONFIG_INTERVAL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)
-      );
 }
 
 static void
 gst_rtp_theora_pay_init (GstRtpTheoraPay * rtptheorapay,
     GstRtpTheoraPayClass * klass)
 {
-  rtptheorapay->last_config = GST_CLOCK_TIME_NONE;
-}
-
-static void
-gst_rtp_theora_pay_clear_packet (GstRtpTheoraPay * rtptheorapay)
-{
-  if (rtptheorapay->packet)
-    gst_buffer_unref (rtptheorapay->packet);
-  rtptheorapay->packet = NULL;
-}
-
-static void
-gst_rtp_theora_pay_cleanup (GstRtpTheoraPay * rtptheorapay)
-{
-  g_list_foreach (rtptheorapay->headers, (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (rtptheorapay->headers);
-  rtptheorapay->headers = NULL;
-
-  gst_rtp_theora_pay_clear_packet (rtptheorapay);
-
-  if (rtptheorapay->config_data)
-    g_free (rtptheorapay->config_data);
-  rtptheorapay->config_data = NULL;
-  rtptheorapay->last_config = GST_CLOCK_TIME_NONE;
 }
 
 static gboolean
@@ -213,7 +163,6 @@ gst_rtp_theora_pay_init_packet (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
       gst_rtp_buffer_new_allocate_len (GST_BASE_RTP_PAYLOAD_MTU
       (rtptheorapay), 0, 0);
   gst_rtp_theora_pay_reset_packet (rtptheorapay, TDT);
-
   GST_BUFFER_TIMESTAMP (rtptheorapay->packet) = timestamp;
 }
 
@@ -254,8 +203,6 @@ gst_rtp_theora_pay_flush_packet (GstRtpTheoraPay * rtptheorapay)
   hlen = gst_rtp_buffer_calc_header_len (0);
   GST_BUFFER_SIZE (rtptheorapay->packet) = hlen + rtptheorapay->payload_pos;
 
-  GST_BUFFER_DURATION (rtptheorapay->packet) = rtptheorapay->payload_duration;
-
   /* push, this gives away our ref to the packet, so clear it. */
   ret =
       gst_basertppayload_push (GST_BASE_RTP_PAYLOAD (rtptheorapay),
@@ -265,23 +212,45 @@ gst_rtp_theora_pay_flush_packet (GstRtpTheoraPay * rtptheorapay)
   return ret;
 }
 
+static gchar *
+encode_base64 (const guint8 * in, guint size, guint * len)
+{
+  gchar *ret, *d;
+  static const gchar *v =
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+  *len = ((size + 2) / 3) * 4;
+  d = ret = (gchar *) g_malloc (*len + 1);
+  for (; size; in += 3) {       /* process tuplets */
+    *d++ = v[in[0] >> 2];       /* byte 1: high 6 bits (1) */
+    /* byte 2: low 2 bits (1), high 4 bits (2) */
+    *d++ = v[((in[0] << 4) + (--size ? (in[1] >> 4) : 0)) & 0x3f];
+    /* byte 3: low 4 bits (2), high 2 bits (3) */
+    *d++ = size ? v[((in[1] << 2) + (--size ? (in[2] >> 6) : 0)) & 0x3f] : '=';
+    /* byte 4: low 6 bits (3) */
+    *d++ = size ? v[in[2] & 0x3f] : '=';
+    if (size)
+      size--;                   /* count third character if processed */
+  }
+  *d = '\0';                    /* tie off string */
+
+  return ret;                   /* return the resulting string */
+}
+
 static gboolean
 gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
 {
   GstRtpTheoraPay *rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
   GList *walk;
-  guint length, size, n_headers, configlen, extralen;
+  guint length, size, n_headers, configlen;
   gchar *wstr, *hstr, *configuration;
   guint8 *data, *config;
   guint32 ident;
-  gboolean res;
 
   GST_DEBUG_OBJECT (rtptheorapay, "finish headers");
 
-  if (!rtptheorapay->headers) {
-    GST_DEBUG_OBJECT (rtptheorapay, "We need 2 headers but have none");
+  if (!rtptheorapay->headers)
     goto no_headers;
-  }
 
   /* +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    * |                     Number of packed headers                  |
@@ -331,10 +300,8 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   length = 0;
   n_headers = 0;
   ident = fnv1_hash_32_new ();
-  extralen = 1;
   for (walk = rtptheorapay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
-
     guint bsize;
 
     bsize = GST_BUFFER_SIZE (buf);
@@ -346,7 +313,6 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
     if (g_list_next (walk)) {
       do {
         size++;
-        extralen++;
         bsize >>= 7;
       } while (bsize);
     }
@@ -385,9 +351,7 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   /* store length for each header */
   for (walk = rtptheorapay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
-
     guint bsize, size, temp;
-    guint flag;
 
     /* only need to store the length when it's not the last header */
     if (!g_list_next (walk))
@@ -405,12 +369,10 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
 
     bsize = GST_BUFFER_SIZE (buf);
     /* write the size backwards */
-    flag = 0;
     while (size) {
       size--;
-      data[size] = (bsize & 0x7f) | flag;
+      data[size] = bsize & 0x7f;
       bsize >>= 7;
-      flag = 0x80;
     }
     data += temp;
   }
@@ -424,22 +386,14 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   }
 
   /* serialize to base64 */
-  configuration = g_base64_encode (config, configlen);
-
-  /* store for later re-sending */
-  rtptheorapay->config_size = configlen - 4 - 3 - 2;
-  rtptheorapay->config_data = g_malloc (rtptheorapay->config_size);
-  rtptheorapay->config_extra_len = extralen;
-  memcpy (rtptheorapay->config_data, config + 4 + 3 + 2,
-      rtptheorapay->config_size);
-
+  configuration = encode_base64 (config, configlen, &size);
   g_free (config);
 
   /* configure payloader settings */
   wstr = g_strdup_printf ("%d", rtptheorapay->width);
   hstr = g_strdup_printf ("%d", rtptheorapay->height);
   gst_basertppayload_set_options (basepayload, "video", TRUE, "THEORA", 90000);
-  res = gst_basertppayload_set_outcaps (basepayload,
+  gst_basertppayload_set_outcaps (basepayload,
       "sampling", G_TYPE_STRING, "YCbCr-4:2:0",
       "width", G_TYPE_STRING, wstr,
       "height", G_TYPE_STRING, hstr,
@@ -452,7 +406,7 @@ gst_rtp_theora_pay_finish_headers (GstBaseRTPPayload * basepayload)
   g_free (hstr);
   g_free (configuration);
 
-  return res;
+  return TRUE;
 
   /* ERRORS */
 no_headers:
@@ -520,19 +474,70 @@ invalid_version:
 }
 
 static GstFlowReturn
-gst_rtp_theora_pay_payload_buffer (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
-    guint8 * data, guint size, GstClockTime timestamp, GstClockTime duration,
-    guint not_in_length)
+gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
+    GstBuffer * buffer)
 {
-  GstFlowReturn ret = GST_FLOW_OK;
-  guint newsize;
+  GstRtpTheoraPay *rtptheorapay;
+  GstFlowReturn ret;
+  guint size, newsize;
+  guint8 *data;
   guint packet_len;
-  GstClockTime newduration;
+  GstClockTime duration, newduration, timestamp;
   gboolean flush;
+  guint8 TDT;
   guint plen;
   guint8 *ppos, *payload;
   gboolean fragmented;
 
+  rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
+
+  size = GST_BUFFER_SIZE (buffer);
+  data = GST_BUFFER_DATA (buffer);
+  duration = GST_BUFFER_DURATION (buffer);
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+
+  GST_DEBUG_OBJECT (rtptheorapay, "size %u, duration %" GST_TIME_FORMAT,
+      size, GST_TIME_ARGS (duration));
+
+  if (G_UNLIKELY (size < 1 || size > 0xffff))
+    goto wrong_size;
+
+  /* find packet type */
+  if (data[0] & 0x80) {
+    /* header */
+    if (data[0] == 0x80) {
+      /* identification, we need to parse this in order to get the clock rate.
+       */
+      if (G_UNLIKELY (!gst_rtp_theora_pay_parse_id (basepayload, data, size)))
+        goto parse_id_failed;
+      TDT = 1;
+    } else if (data[0] == 0x81) {
+      /* comment */
+      TDT = 2;
+    } else if (data[0] == 0x82) {
+      /* setup */
+      TDT = 1;
+    } else
+      goto unknown_header;
+  } else
+    /* data */
+    TDT = 0;
+
+  if (rtptheorapay->need_headers) {
+    /* we need to collect the headers and construct a config string from them */
+    if (TDT != 0) {
+      GST_DEBUG_OBJECT (rtptheorapay, "collecting header, buffer %p", buffer);
+      /* append header to the list of headers */
+      rtptheorapay->headers = g_list_append (rtptheorapay->headers, buffer);
+      ret = GST_FLOW_OK;
+      goto done;
+    } else {
+      if (!gst_rtp_theora_pay_finish_headers (basepayload))
+        goto header_error;
+      rtptheorapay->need_headers = FALSE;
+    }
+  }
+
   /* size increases with packet length and 2 bytes size eader. */
   newduration = rtptheorapay->payload_duration;
   if (duration != GST_CLOCK_TIME_NONE)
@@ -542,8 +547,7 @@ gst_rtp_theora_pay_payload_buffer (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
   packet_len = gst_rtp_buffer_calc_packet_len (newsize, 0, 0);
 
   /* check buffer filled against length and max latency */
-  flush = gst_basertppayload_is_filled (GST_BASE_RTP_PAYLOAD (rtptheorapay),
-      packet_len, newduration);
+  flush = gst_basertppayload_is_filled (basepayload, packet_len, newduration);
   /* we can store up to 15 theora packets in one RTP packet. */
   flush |= (rtptheorapay->payload_pkts == 15);
   /* flush if we have a new TDT */
@@ -561,22 +565,19 @@ gst_rtp_theora_pay_payload_buffer (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
   ppos = payload + rtptheorapay->payload_pos;
   fragmented = FALSE;
 
+  ret = GST_FLOW_OK;
+
   /* put buffer in packet, it either fits completely or needs to be fragmented
    * over multiple RTP packets. */
-  do {
+  while (size) {
     plen = MIN (rtptheorapay->payload_left - 2, size);
 
     GST_DEBUG_OBJECT (rtptheorapay, "append %u bytes", plen);
 
     /* data is copied in the payload with a 2 byte length header */
-    ppos[0] = ((plen - not_in_length) >> 8) & 0xff;
-    ppos[1] = ((plen - not_in_length) & 0xff);
-    if (plen)
-      memcpy (&ppos[2], data, plen);
-
-    /* only first (only) configuration cuts length field */
-    /* NOTE: spec (if any) is not clear on this ... */
-    not_in_length = 0;
+    ppos[0] = (plen >> 8) & 0xff;
+    ppos[1] = (plen & 0xff);
+    memcpy (&ppos[2], data, plen);
 
     size -= plen;
     data += plen;
@@ -618,128 +619,7 @@ gst_rtp_theora_pay_payload_buffer (GstRtpTheoraPay * rtptheorapay, guint8 TDT,
       if (duration != GST_CLOCK_TIME_NONE)
         rtptheorapay->payload_duration += duration;
     }
-  } while (size);
-
-  return ret;
-}
-
-static GstFlowReturn
-gst_rtp_theora_pay_handle_buffer (GstBaseRTPPayload * basepayload,
-    GstBuffer * buffer)
-{
-  GstRtpTheoraPay *rtptheorapay;
-  GstFlowReturn ret;
-  guint size;
-  guint8 *data;
-  GstClockTime duration, timestamp;
-  guint8 TDT;
-  gboolean keyframe = FALSE;
-
-  rtptheorapay = GST_RTP_THEORA_PAY (basepayload);
-
-  size = GST_BUFFER_SIZE (buffer);
-  data = GST_BUFFER_DATA (buffer);
-  duration = GST_BUFFER_DURATION (buffer);
-  timestamp = GST_BUFFER_TIMESTAMP (buffer);
-
-  GST_DEBUG_OBJECT (rtptheorapay, "size %u, duration %" GST_TIME_FORMAT,
-      size, GST_TIME_ARGS (duration));
-
-  if (G_UNLIKELY (size > 0xffff))
-    goto wrong_size;
-
-  /* find packet type */
-  if (size == 0) {
-    TDT = 0;
-    keyframe = FALSE;
-  } else if (data[0] & 0x80) {
-    /* header */
-    if (data[0] == 0x80) {
-      /* identification, we need to parse this in order to get the clock rate.
-       */
-      if (G_UNLIKELY (!gst_rtp_theora_pay_parse_id (basepayload, data, size)))
-        goto parse_id_failed;
-      TDT = 1;
-    } else if (data[0] == 0x81) {
-      /* comment */
-      TDT = 2;
-    } else if (data[0] == 0x82) {
-      /* setup */
-      TDT = 1;
-    } else
-      goto unknown_header;
-  } else {
-    /* data */
-    TDT = 0;
-    keyframe = ((data[0] & 0x40) == 0);
   }
-
-  if (rtptheorapay->need_headers) {
-    /* we need to collect the headers and construct a config string from them */
-    if (TDT != 0) {
-      GST_DEBUG_OBJECT (rtptheorapay, "collecting header, buffer %p", buffer);
-      /* append header to the list of headers */
-      rtptheorapay->headers = g_list_append (rtptheorapay->headers, buffer);
-      ret = GST_FLOW_OK;
-      goto done;
-    } else {
-      if (!gst_rtp_theora_pay_finish_headers (basepayload))
-        goto header_error;
-      rtptheorapay->need_headers = FALSE;
-    }
-  }
-
-  /* there is a config request, see if we need to insert it */
-  if (keyframe && (rtptheorapay->config_interval > 0) &&
-      rtptheorapay->config_data) {
-    gboolean send_config = FALSE;
-
-    if (rtptheorapay->last_config != -1) {
-      guint64 diff;
-
-      GST_LOG_OBJECT (rtptheorapay,
-          "now %" GST_TIME_FORMAT ", last VOP-I %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (timestamp), GST_TIME_ARGS (rtptheorapay->last_config));
-
-      /* calculate diff between last config in milliseconds */
-      if (timestamp > rtptheorapay->last_config) {
-        diff = timestamp - rtptheorapay->last_config;
-      } else {
-        diff = 0;
-      }
-
-      GST_DEBUG_OBJECT (rtptheorapay,
-          "interval since last config %" GST_TIME_FORMAT, GST_TIME_ARGS (diff));
-
-      /* bigger than interval, queue config */
-      /* FIXME should convert timestamps to running time */
-      if (GST_TIME_AS_SECONDS (diff) >= rtptheorapay->config_interval) {
-        GST_DEBUG_OBJECT (rtptheorapay, "time to send config");
-        send_config = TRUE;
-      }
-    } else {
-      /* no known previous config time, send now */
-      GST_DEBUG_OBJECT (rtptheorapay, "no previous config time, send now");
-      send_config = TRUE;
-    }
-
-    if (send_config) {
-      /* we need to send config now first */
-      /* different TDT type forces flush */
-      gst_rtp_theora_pay_payload_buffer (rtptheorapay, 1,
-          rtptheorapay->config_data, rtptheorapay->config_size,
-          timestamp, GST_CLOCK_TIME_NONE, rtptheorapay->config_extra_len);
-
-      if (timestamp != -1) {
-        rtptheorapay->last_config = timestamp;
-      }
-    }
-  }
-
-  ret = gst_rtp_theora_pay_payload_buffer (rtptheorapay, TDT, data, size,
-      timestamp, duration, 0);
-  gst_buffer_unref (buffer);
-
 done:
   return ret;
 
@@ -747,111 +627,30 @@ done:
 wrong_size:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
-        ("Invalid packet size (%d <= 0xffff)", size), (NULL));
-    gst_buffer_unref (buffer);
+        ("Invalid packet size (1 < %d <= 0xffff)", size), (NULL));
     return GST_FLOW_OK;
   }
 parse_id_failed:
   {
-    gst_buffer_unref (buffer);
     return GST_FLOW_ERROR;
   }
 unknown_header:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
         (NULL), ("Ignoring unknown header received"));
-    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 header_error:
   {
     GST_ELEMENT_WARNING (rtptheorapay, STREAM, DECODE,
         (NULL), ("Error initializing header config"));
-    gst_buffer_unref (buffer);
     return GST_FLOW_OK;
   }
 }
 
-static gboolean
-gst_rtp_theora_pay_handle_event (GstPad * pad, GstEvent * event)
-{
-  GstRtpTheoraPay *rtptheorapay = GST_RTP_THEORA_PAY (GST_PAD_PARENT (pad));
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_FLUSH_STOP:
-      gst_rtp_theora_pay_clear_packet (rtptheorapay);
-      break;
-    default:
-      break;
-  }
-  /* false to let parent handle event as well */
-  return FALSE;
-}
-
-static GstStateChangeReturn
-gst_rtp_theora_pay_change_state (GstElement * element,
-    GstStateChange transition)
-{
-  GstRtpTheoraPay *rtptheorapay;
-
-  GstStateChangeReturn ret;
-
-  rtptheorapay = GST_RTP_THEORA_PAY (element);
-
-  switch (transition) {
-    default:
-      break;
-  }
-
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_rtp_theora_pay_cleanup (rtptheorapay);
-      break;
-    default:
-      break;
-  }
-  return ret;
-}
-
-static void
-gst_rtp_theora_pay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpTheoraPay *rtptheorapay;
-
-  rtptheorapay = GST_RTP_THEORA_PAY (object);
-
-  switch (prop_id) {
-    case PROP_CONFIG_INTERVAL:
-      rtptheorapay->config_interval = g_value_get_uint (value);
-      break;
-    default:
-      break;
-  }
-}
-
-static void
-gst_rtp_theora_pay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpTheoraPay *rtptheorapay;
-
-  rtptheorapay = GST_RTP_THEORA_PAY (object);
-
-  switch (prop_id) {
-    case PROP_CONFIG_INTERVAL:
-      g_value_set_uint (value, rtptheorapay->config_interval);
-      break;
-    default:
-      break;
-  }
-}
-
 gboolean
 gst_rtp_theora_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtptheorapay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_THEORA_PAY);
+      GST_RANK_NONE, GST_TYPE_RTP_THEORA_PAY);
 }
