@@ -48,7 +48,10 @@
 G_BEGIN_DECLS
 
 #define RTSP_CHECK(stmt, label)  \
-if (G_UNLIKELY ((res = (stmt)) != RTSP_OK)) goto label
+G_STMT_START { \
+  if (G_UNLIKELY ((res = (stmt)) != RTSP_OK)) \
+    goto label; \
+} G_STMT_END
 
 typedef enum {
   RTSP_OK          =  0,
@@ -66,8 +69,9 @@ typedef enum {
   RTSP_EEOF        = -11,
   RTSP_ENET        = -12,
   RTSP_ENOTIP      = -13,
+  RTSP_ETIMEOUT    = -14,
 
-  RTSP_ELAST       = -14,
+  RTSP_ELAST       = -15,
 } RTSPResult;
 
 typedef enum {
@@ -80,10 +84,16 @@ typedef enum {
   RTSP_STATE_INVALID,
   RTSP_STATE_INIT,
   RTSP_STATE_READY,
+  RTSP_STATE_SEEKING,
   RTSP_STATE_PLAYING,
   RTSP_STATE_RECORDING,
 } RTSPState;
 
+typedef enum {
+  RTSP_VERSION_INVALID = 0x00,
+  RTSP_VERSION_1_0     = 0x10,
+} RTSPVersion;
+
 typedef enum {
   RTSP_INVALID          = 0,
   RTSP_DESCRIBE         = (1 <<  0),
@@ -99,7 +109,19 @@ typedef enum {
   RTSP_TEARDOWN         = (1 << 10),
 } RTSPMethod;
 
+/* Authentication methods, ordered by strength */
 typedef enum {
+  RTSP_AUTH_NONE    = 0x00,
+  RTSP_AUTH_BASIC   = 0x01,
+  RTSP_AUTH_DIGEST  = 0x02
+} RTSPAuthMethod;
+
+/* Strongest available authentication method */
+#define RTSP_AUTH_MAX RTSP_AUTH_DIGEST
+
+typedef enum {
+  RTSP_HDR_INVALID,
+
   /*
    * R = Request
    * r = response
@@ -215,9 +237,9 @@ typedef enum {
 gchar*          rtsp_strresult          (RTSPResult result);
 
 const gchar*    rtsp_method_as_text     (RTSPMethod method);
+const gchar*    rtsp_version_as_text    (RTSPVersion version);
 const gchar*    rtsp_header_as_text     (RTSPHeaderField field);
 const gchar*    rtsp_status_as_text     (RTSPStatusCode code);
-const gchar*    rtsp_status_to_string   (RTSPStatusCode code);
 
 RTSPHeaderField rtsp_find_header_field  (gchar *header);
 RTSPMethod      rtsp_find_method        (gchar *method);
