@@ -36,7 +36,7 @@
 #include "gstudp-marshal.h"
 #include "gstmultiudpsink.h"
 
-GST_DEBUG_CATEGORY (multiudpsink_debug);
+GST_DEBUG_CATEGORY_STATIC (multiudpsink_debug);
 #define GST_CAT_DEFAULT (multiudpsink_debug)
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -92,6 +92,8 @@ static void gst_multiudpsink_set_property (GObject * object, guint prop_id,
 static void gst_multiudpsink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
+static void free_client (GstUDPClient * client);
+
 static GstElementClass *parent_class = NULL;
 
 static guint gst_multiudpsink_signals[LAST_SIGNAL] = { 0 };
@@ -257,6 +259,8 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
 static void
 gst_multiudpsink_init (GstMultiUDPSink * sink)
 {
+  WSA_STARTUP (sink);
+
   sink->client_lock = g_mutex_new ();
 }
 
@@ -267,8 +271,13 @@ gst_multiudpsink_finalize (GObject * object)
 
   sink = GST_MULTIUDPSINK (object);
 
+  g_list_foreach (sink->clients, (GFunc) free_client, NULL);
+  g_list_free (sink->clients);
+
   g_mutex_free (sink->client_lock);
 
+  WSA_CLEANUP (object);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -370,6 +379,34 @@ gst_multiudpsink_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
+static void
+join_multicast (GstUDPClient * client)
+{
+  unsigned char ttl = 64;
+  unsigned char loop = 1;
+
+  /* Joining the multicast group */
+  /* FIXME, can we use multicast and unicast over the same
+   * socket? if not, search for socket of this multicast group or
+   * create a new one. */
+  if (setsockopt (*(client->sock), IPPROTO_IP, IP_ADD_MEMBERSHIP,
+          &(client->multi_addr), sizeof (client->multi_addr)) < 0)
+    perror ("setsockopt IP_ADD_MEMBERSHIP\n");
+  if (setsockopt (*(client->sock), IPPROTO_IP, IP_MULTICAST_TTL, &ttl,
+          sizeof (ttl)) < 0)
+    perror ("setsockopt IP_MULTICAST_TTL\n");
+  if (setsockopt (*(client->sock), IPPROTO_IP, IP_MULTICAST_LOOP, &loop,
+          sizeof (loop)) < 0)
+    perror ("setsockopt IP_MULTICAST_LOOP\n");
+}
+
+static void
+leave_multicast (GstUDPClient * client)
+{
+  if (setsockopt (*(client->sock), IPPROTO_IP, IP_DROP_MEMBERSHIP,
+          &(client->multi_addr), sizeof (client->multi_addr)) < 0)
+    perror ("setsockopt IP_DROP_MEMBERSHIP\n");
+}
 
 /* create a socket for sending to remote machine */
 static gboolean
@@ -377,6 +414,8 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
 {
   guint bc_val;
   gint ret;
+  GList *clients;
+  GstUDPClient *client;
 
   /* create sender socket */
   if ((sink->sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1)
@@ -391,6 +430,13 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
   sink->bytes_to_serve = 0;
   sink->bytes_served = 0;
 
+  /* look for multicast clients and join multicast groups approptiately */
+
+  for (clients = sink->clients; clients; clients = g_list_next (clients)) {
+    client = (GstUDPClient *) clients->data;
+    if (client->multi_addr.imr_multiaddr.s_addr)
+      join_multicast (client);
+  }
   return TRUE;
 
   /* ERRORS */
@@ -412,7 +458,7 @@ no_broadcast:
 static void
 gst_multiudpsink_close (GstMultiUDPSink * sink)
 {
-  close (sink->sock);
+  CLOSE_SOCKET (sink->sock);
 }
 
 void
@@ -420,7 +466,6 @@ gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
 {
   struct hostent *he;
   struct in_addr addr;
-  struct ip_mreq multi_addr;
   GstUDPClient *client;
   GTimeVal now;
 
@@ -431,6 +476,7 @@ gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
   client->sock = &sink->sock;
 
   memset (&client->theiraddr, 0, sizeof (client->theiraddr));
+  memset (&client->multi_addr, 0, sizeof (client->multi_addr));
   client->theiraddr.sin_family = AF_INET;       /* host byte order */
   client->theiraddr.sin_port = htons (port);    /* short, network byte order */
 
@@ -440,21 +486,20 @@ gst_multiudpsink_add (GstMultiUDPSink * sink, const gchar * host, gint port)
   /* if its an IP address */
   if (inet_aton (host, &addr)) {
     /* check if its a multicast address */
-    if ((ntohl (addr.s_addr) & 0xe0000000) == 0xe0000000) {
+    if ((ntohl (addr.s_addr) & 0xf0000000) == 0xe0000000) {
+      GST_DEBUG_OBJECT (sink, "multicast address detected");
       client->multi_addr.imr_multiaddr.s_addr = addr.s_addr;
       client->multi_addr.imr_interface.s_addr = INADDR_ANY;
 
-      client->theiraddr.sin_addr = multi_addr.imr_multiaddr;
+      client->theiraddr.sin_addr = client->multi_addr.imr_multiaddr;
 
-      /* Joining the multicast group */
-      /* FIXME, can we use multicast and unicast over the same
-       * socket? if not, search for socket of this multicast group or
-       * create a new one. */
-      setsockopt (sink->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, &multi_addr,
-          sizeof (multi_addr));
     } else {
+      GST_DEBUG_OBJECT (sink, "normal address detected");
       client->theiraddr.sin_addr = *((struct in_addr *) &addr);
     }
+    /* if init_send has already been called, set sockopts for multicast */
+    if (*client->sock > 0 && client->multi_addr.imr_multiaddr.s_addr)
+      join_multicast (client);
   }
   /* we dont need to lookup for localhost */
   else if (strcmp (host, "localhost") == 0 && inet_aton ("127.0.0.1", &addr)) {
@@ -526,6 +571,9 @@ gst_multiudpsink_remove (GstMultiUDPSink * sink, const gchar * host, gint port)
   g_get_current_time (&now);
   client->disconnect_time = GST_TIMEVAL_TO_TIME (now);
 
+  if (client->multi_addr.imr_multiaddr.s_addr)
+    leave_multicast (client);
+
   /* Unlock to emit signal before we delete the actual client */
   g_mutex_unlock (sink->client_lock);
   g_signal_emit (G_OBJECT (sink),
