@@ -31,20 +31,20 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/ioctl.h>
+#include <sys/audioio.h>
 
 #include <gst/gst-i18n-plugin.h>
 
 #include "gstsunaudiomixertrack.h"
 
-#define MIXER_DEVICES 3
 #define MASK_BIT_IS_SET(mask, bit) \
   (mask & (1 << bit))
 
 G_DEFINE_TYPE (GstSunAudioMixerTrack, gst_sunaudiomixer_track,
-    GST_TYPE_MIXER_TRACK);
+    GST_TYPE_MIXER_TRACK)
 
-static void
-gst_sunaudiomixer_track_class_init (GstSunAudioMixerTrackClass * klass)
+     static void
+         gst_sunaudiomixer_track_class_init (GstSunAudioMixerTrackClass * klass)
 {
   /* nop */
 }
@@ -52,57 +52,47 @@ gst_sunaudiomixer_track_class_init (GstSunAudioMixerTrackClass * klass)
 static void
 gst_sunaudiomixer_track_init (GstSunAudioMixerTrack * track)
 {
-  track->vol = 0;
+  track->gain = 0;
+  track->balance = AUDIO_MID_BALANCE;
   track->track_num = 0;
 }
 
-static const gchar **labels = NULL;
-
-static void
-fill_labels (void)
-{
-  int i;
-  struct
-  {
-    gchar *given, *wanted;
-  }
-  cases[] = {
-    {
-    "Vol  ", N_("Volume")}
-    , {
-    "Gain ", N_("Gain")}
-    , {
-    "Mon  ", N_("Monitor")}
-    , {
-    NULL, NULL}
-  };
-
-  labels = g_malloc (sizeof (gchar *) * MIXER_DEVICES);
-
-  for (i = 0; i < MIXER_DEVICES; i++) {
-    labels[i] = g_strdup (cases[i].wanted);
-  }
-}
-
 GstMixerTrack *
-gst_sunaudiomixer_track_new (gint track_num, gint max_chans, gint flags)
+gst_sunaudiomixer_track_new (GstSunAudioTrackType track_num,
+    gint max_chans, gint flags)
 {
+  const gchar *labels[] = { N_("Volume"), N_("Gain"), N_("Monitor") };
+
   GstSunAudioMixerTrack *sunaudiotrack;
   GstMixerTrack *track;
+  GObjectClass *klass;
+  const gchar *untranslated_label;
+  gint volume;
 
-  if (!labels)
-    fill_labels ();
+  if ((guint) track_num < G_N_ELEMENTS (labels))
+    untranslated_label = labels[track_num];
+  else
+    untranslated_label = NULL;
+
+  /* FIXME: remove this check once we depend on -base >= 0.10.12.1 */
+  klass = G_OBJECT_CLASS (g_type_class_ref (GST_TYPE_SUNAUDIO_MIXER_TRACK));
+  if (g_object_class_find_property (klass, "untranslated-label")) {
+    sunaudiotrack = g_object_new (GST_TYPE_SUNAUDIO_MIXER_TRACK,
+        "untranslated-label", untranslated_label, NULL);
+  } else {
+    sunaudiotrack = g_object_new (GST_TYPE_SUNAUDIO_MIXER_TRACK, NULL);
+  }
+  g_type_class_unref (klass);
 
-  sunaudiotrack = g_object_new (GST_TYPE_SUNAUDIO_MIXER_TRACK, NULL);
   track = GST_MIXER_TRACK (sunaudiotrack);
-  track->label = g_strdup (labels[track_num]);
+  track->label = g_strdup (_(untranslated_label));
   track->num_channels = max_chans;
   track->flags = flags;
   track->min_volume = 0;
   track->max_volume = 100;
   sunaudiotrack->track_num = track_num;
-
-  sunaudiotrack->vol = (0 & 0xff);
+  sunaudiotrack->gain = (0 & 0xff);
+  sunaudiotrack->balance = AUDIO_MID_BALANCE;
 
   return track;
 }
