@@ -78,7 +78,12 @@ enum
   PROP_0,
   PROP_DISPLAY_NAME,
   PROP_SCREEN_NUM,
-  PROP_SHOW_POINTER
+  PROP_SHOW_POINTER,
+  PROP_USE_DAMAGE,
+  PROP_STARTX,
+  PROP_STARTY,
+  PROP_ENDX,
+  PROP_ENDY
 };
 
 GST_BOILERPLATE (GstXImageSrc, gst_ximage_src, GstPushSrc, GST_TYPE_PUSH_SRC);
@@ -122,6 +127,13 @@ gst_ximage_src_open_display (GstXImageSrc * s, const gchar * name)
 
   g_mutex_lock (s->x_lock);
   s->xcontext = ximageutil_xcontext_get (GST_ELEMENT (s), name);
+  if (s->xcontext == NULL) {
+    g_mutex_unlock (s->x_lock);
+    GST_ELEMENT_ERROR (s, RESOURCE, OPEN_READ,
+        ("Could not open X display for reading"),
+        ("NULL returned from getting xcontext"));
+    return FALSE;
+  }
   s->width = s->xcontext->width;
   s->height = s->xcontext->height;
 
@@ -152,8 +164,7 @@ gst_ximage_src_open_display (GstXImageSrc * s, const gchar * name)
     if (XDamageQueryExtension (s->xcontext->disp, &s->damage_event_base,
             &error_base)) {
       s->damage =
-          XDamageCreate (s->xcontext->disp, s->xwindow,
-          XDamageReportRawRectangles);
+          XDamageCreate (s->xcontext->disp, s->xwindow, XDamageReportNonEmpty);
       if (s->damage != None) {
         s->damage_region = XFixesCreateRegion (s->xcontext->disp, NULL, 0);
         if (s->damage_region != None) {
@@ -193,8 +204,10 @@ gst_ximage_src_start (GstBaseSrc * basesrc)
   GstXImageSrc *s = GST_XIMAGE_SRC (basesrc);
 
   s->last_frame_no = -1;
-
-  return gst_ximage_src_open_display (s, NULL);
+#ifdef HAVE_XDAMAGE
+  s->last_ximage = NULL;
+#endif
+  return gst_ximage_src_open_display (s, s->display_name);
 }
 
 static gboolean
@@ -260,7 +273,7 @@ composite_pixel (GstXContext * xcontext, guchar * dest, guchar * src)
       color = *dest;
       break;
     case 16:
-      color = GUINT16_FROM_LE (*(guint32 *) (dest));
+      color = GUINT16_FROM_LE (*(guint16 *) (dest));
       break;
     case 32:
       color = GUINT32_FROM_LE (*(guint32 *) (dest));
@@ -327,7 +340,6 @@ static GstXImageSrcBuffer *
 gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 {
   GstXImageSrcBuffer *ximage = NULL;
-  GstCaps *caps = NULL;
 
   g_mutex_lock (ximagesrc->pool_lock);
   while (ximagesrc->buffer_pool != NULL) {
@@ -345,6 +357,7 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 
   if (ximage == NULL) {
     GstXContext *xcontext;
+    GstCaps *caps = NULL;
 
     GST_DEBUG_OBJECT (ximagesrc, "creating image (%dx%d)",
         ximagesrc->width, ximagesrc->height);
@@ -362,6 +375,7 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 
     xcontext = ximagesrc->xcontext;
 
+
     caps = gst_caps_new_simple ("video/x-raw-rgb",
         "bpp", G_TYPE_INT, xcontext->bpp,
         "depth", G_TYPE_INT, xcontext->depth,
@@ -369,8 +383,8 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
         "red_mask", G_TYPE_INT, xcontext->r_mask_output,
         "green_mask", G_TYPE_INT, xcontext->g_mask_output,
         "blue_mask", G_TYPE_INT, xcontext->b_mask_output,
-        "width", G_TYPE_INT, xcontext->width,
-        "height", G_TYPE_INT, xcontext->height,
+        "width", G_TYPE_INT, ximagesrc->width,
+        "height", G_TYPE_INT, ximagesrc->height,
         "framerate", GST_TYPE_FRACTION, ximagesrc->fps_n, ximagesrc->fps_d,
         "pixel-aspect-ratio", GST_TYPE_FRACTION,
         gst_value_get_fraction_numerator (xcontext->par),
@@ -378,50 +392,114 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 
     gst_buffer_set_caps (GST_BUFFER (ximage), caps);
     g_mutex_unlock (ximagesrc->x_lock);
+
+    gst_caps_unref (caps);
   }
 
   g_return_val_if_fail (GST_IS_XIMAGE_SRC (ximagesrc), NULL);
-
 #ifdef HAVE_XDAMAGE
-  if (ximagesrc->have_xdamage) {
+  if (ximagesrc->have_xdamage && ximagesrc->use_damage &&
+      ximagesrc->last_ximage != NULL) {
     XEvent ev;
 
+    /* have_frame is TRUE when either the entire screen has been
+     * grabbed or when the last image has been copied */
+    gboolean have_frame = FALSE;
+
     GST_DEBUG_OBJECT (ximagesrc, "Retrieving screen using XDamage");
 
     do {
       XNextEvent (ximagesrc->xcontext->disp, &ev);
+
       if (ev.type == ximagesrc->damage_event_base + XDamageNotify) {
-        XDamageNotifyEvent *dev = (XDamageNotifyEvent *) & ev;
+        XserverRegion parts;
+        XRectangle *rects;
+        int nrects;
 
-#ifdef HAVE_XSHM
-        if (ximagesrc->xcontext->use_xshm &&
-            dev->area.width == ximagesrc->width &&
-            dev->area.height == ximagesrc->height) {
-          GST_DEBUG_OBJECT (ximagesrc, "Entire screen was damaged");
-          XShmGetImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-              ximage->ximage, 0, 0, AllPlanes);
-          /* No need to collect more events */
-          while (XPending (ximagesrc->xcontext->disp)) {
-            XNextEvent (ximagesrc->xcontext->disp, &ev);
+        parts = XFixesCreateRegion (ximagesrc->xcontext->disp, 0, 0);
+        XDamageSubtract (ximagesrc->xcontext->disp, ximagesrc->damage, None,
+            parts);
+        /* Now copy out all of the damaged rectangles. */
+        rects = XFixesFetchRegion (ximagesrc->xcontext->disp, parts, &nrects);
+        if (rects != NULL) {
+          int i;
+
+          if (!have_frame) {
+            GST_LOG_OBJECT (ximagesrc,
+                "Copying from last frame ximage->size: %d",
+                GST_BUFFER_SIZE (GST_BUFFER (ximage)));
+            memcpy (GST_BUFFER_DATA (GST_BUFFER (ximage)),
+                GST_BUFFER_DATA (GST_BUFFER (ximagesrc->last_ximage)),
+                GST_BUFFER_SIZE (GST_BUFFER (ximage)));
+            have_frame = TRUE;
           }
-          break;
-        } else
-#endif
-        {
-          GST_LOG_OBJECT (ximagesrc,
-              "Retrieving damaged sub-region @ %d,%d size %dx%d",
-              dev->area.x, dev->area.y, dev->area.width, dev->area.height);
+          for (i = 0; i < nrects; i++) {
+            GST_LOG_OBJECT (ximagesrc,
+                "Damaged sub-region @ %d,%d size %dx%d reported",
+                rects[i].x, rects[i].y, rects[i].width, rects[i].height);
 
-          XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-              dev->area.x, dev->area.y,
-              dev->area.width, dev->area.height,
-              AllPlanes, ZPixmap, ximage->ximage, dev->area.x, dev->area.y);
+            /* if we only want a small area, clip this damage region to
+             * area we want */
+            if (ximagesrc->endx > ximagesrc->startx &&
+                ximagesrc->endy > ximagesrc->starty) {
+              /* see if damage area intersects */
+              if (rects[i].x + rects[i].width < ximagesrc->startx ||
+                  rects[i].x > ximagesrc->endx) {
+                /* trivial reject */
+              } else if (rects[i].y + rects[i].height < ximagesrc->starty ||
+                  rects[i].y > ximagesrc->endy) {
+                /* trivial reject */
+              } else {
+                /* find intersect region */
+                int startx, starty, width, height;
+
+                startx = (rects[i].x < ximagesrc->startx) ? ximagesrc->startx :
+                    rects[i].x;
+                starty = (rects[i].y < ximagesrc->starty) ? ximagesrc->starty :
+                    rects[i].y;
+                width = (rects[i].x + rects[i].width < ximagesrc->endx) ?
+                    rects[i].x + rects[i].width - startx :
+                    ximagesrc->endx - startx;
+                height = (rects[i].y + rects[i].height < ximagesrc->endy) ?
+                    rects[i].y + rects[i].height - starty : ximagesrc->endy -
+                    starty;
+
+                GST_LOG_OBJECT (ximagesrc,
+                    "Retrieving damaged sub-region @ %d,%d size %dx%d as intersect region",
+                    startx, starty, width, height);
+                XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+                    startx, starty, width, height, AllPlanes, ZPixmap,
+                    ximage->ximage, startx - ximagesrc->startx,
+                    starty - ximagesrc->starty);
+              }
+            } else {
+
+              GST_LOG_OBJECT (ximagesrc,
+                  "Retrieving damaged sub-region @ %d,%d size %dx%d",
+                  rects[i].x, rects[i].y, rects[i].width, rects[i].height);
+
+              XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+                  rects[i].x, rects[i].y,
+                  rects[i].width, rects[i].height,
+                  AllPlanes, ZPixmap, ximage->ximage, rects[i].x, rects[i].y);
+            }
+          }
+          free (rects);
         }
       }
     } while (XPending (ximagesrc->xcontext->disp));
-    XDamageSubtract (ximagesrc->xcontext->disp, ximagesrc->damage, None, None);
+    if (!have_frame) {
+      GST_LOG_OBJECT (ximagesrc,
+          "Copying from last frame ximage->size: %d",
+          GST_BUFFER_SIZE (GST_BUFFER (ximage)));
+      memcpy (GST_BUFFER_DATA (GST_BUFFER (ximage)),
+          GST_BUFFER_DATA (GST_BUFFER (ximagesrc->last_ximage)),
+          GST_BUFFER_SIZE (GST_BUFFER (ximage)));
+      have_frame = TRUE;
+    }
 #ifdef HAVE_XFIXES
-    /* re-get area where last mouse pointer was */
+    /* re-get area where last mouse pointer was  but only if in our clipping
+     * bounds */
     if (ximagesrc->cursor_image) {
       gint x, y, width, height;
 
@@ -435,16 +513,46 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
         x = 0;
       if (y < 0)
         y = 0;
-      if (x + width > ximagesrc->width)
-        width = ximagesrc->width - x;
-      if (y + height > ximagesrc->height)
-        height = ximagesrc->height - y;
+      if (x + width > ximagesrc->xcontext->width)
+        width = ximagesrc->xcontext->width - x;
+      if (y + height > ximagesrc->xcontext->height)
+        height = ximagesrc->xcontext->height - y;
       g_assert (x >= 0);
       g_assert (y >= 0);
+      GST_DEBUG_OBJECT (ximagesrc,
+          "Cursor was at (%d,%d) width: %d, height: %d and our range is: (%d,%d) - (%d,%d)",
+          x, y, width, height, ximagesrc->startx, ximagesrc->starty,
+          ximagesrc->endx, ximagesrc->endy);
+      /* only get where cursor last was, if it is in our range */
+      if (ximagesrc->endx > ximagesrc->startx &&
+          ximagesrc->endy > ximagesrc->starty) {
+        /* check bounds */
+        if (x + width < ximagesrc->startx || x > ximagesrc->endx) {
+          /* trivial reject */
+        } else if (y + height < ximagesrc->starty || y > ximagesrc->endy) {
+          /* trivial reject */
+        } else {
+          /* find intersect region */
+          int startx, starty, iwidth, iheight;
 
-      GST_DEBUG_OBJECT (ximagesrc, "Removing cursor from %d,%d", x, y);
-      XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-          x, y, width, height, AllPlanes, ZPixmap, ximage->ximage, x, y);
+          startx = (x < ximagesrc->startx) ? ximagesrc->startx : x;
+          starty = (y < ximagesrc->starty) ? ximagesrc->starty : y;
+          iwidth = (x + width < ximagesrc->endx) ?
+              x + width - startx : ximagesrc->endx - startx;
+          iheight = (y + height < ximagesrc->endy) ?
+              y + height - starty : ximagesrc->endy - starty;
+          GST_DEBUG_OBJECT (ximagesrc, "Removing cursor from %d,%d", x, y);
+          XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+              startx, starty, iwidth, iheight, AllPlanes, ZPixmap,
+              ximage->ximage, startx - ximagesrc->startx,
+              starty - ximagesrc->starty);
+        }
+      } else {
+
+        GST_DEBUG_OBJECT (ximagesrc, "Removing cursor from %d,%d", x, y);
+        XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+            x, y, width, height, AllPlanes, ZPixmap, ximage->ximage, x, y);
+      }
     }
 #endif
 
@@ -456,14 +564,15 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
     if (ximagesrc->xcontext->use_xshm) {
       GST_DEBUG_OBJECT (ximagesrc, "Retrieving screen using XShm");
       XShmGetImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-          ximage->ximage, 0, 0, AllPlanes);
+          ximage->ximage, ximagesrc->startx, ximagesrc->starty, AllPlanes);
 
     } else
 #endif /* HAVE_XSHM */
     {
       GST_DEBUG_OBJECT (ximagesrc, "Retrieving screen using XGetImage");
       ximage->ximage = XGetImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-          0, 0, ximagesrc->width, ximagesrc->height, AllPlanes, ZPixmap);
+          ximagesrc->startx, ximagesrc->starty, ximagesrc->width,
+          ximagesrc->height, AllPlanes, ZPixmap);
     }
 #ifdef HAVE_XDAMAGE
   }
@@ -477,38 +586,92 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
     ximagesrc->cursor_image = XFixesGetCursorImage (ximagesrc->xcontext->disp);
     if (ximagesrc->cursor_image != NULL) {
       int cx, cy, i, j, count;
+      int startx, starty, iwidth, iheight;
+      gboolean clipped = FALSE;
+      gboolean cursor_in_image = TRUE;
 
       cx = ximagesrc->cursor_image->x - ximagesrc->cursor_image->xhot;
+      if (cx < 0)
+        cx = 0;
       cy = ximagesrc->cursor_image->y - ximagesrc->cursor_image->yhot;
+      if (cy < 0)
+        cy = 0;
       count = ximagesrc->cursor_image->width * ximagesrc->cursor_image->height;
-      for (i = 0; i < count; i++)
-        ximagesrc->cursor_image->pixels[i] =
-            GUINT_TO_LE (ximagesrc->cursor_image->pixels[i]);
 
-      /* copy those pixels across */
-      for (j = cy;
-          j < cy + ximagesrc->cursor_image->height && j < ximagesrc->height;
-          j++) {
-        for (i = cx;
-            i < cx + ximagesrc->cursor_image->width && i < ximagesrc->width;
-            i++) {
-          guint8 *src, *dest;
+      /* only get where cursor last was, if it is in our range */
+      if (ximagesrc->endx > ximagesrc->startx &&
+          ximagesrc->endy > ximagesrc->starty) {
+        /* check bounds */
+        if (cx + ximagesrc->cursor_image->width < ximagesrc->startx ||
+            cx > ximagesrc->endx) {
+          /* trivial reject */
+          cursor_in_image = FALSE;
+        } else if (cy + ximagesrc->cursor_image->height < ximagesrc->starty ||
+            cy > ximagesrc->endy) {
+          /* trivial reject */
+          cursor_in_image = FALSE;
+        } else {
+          /* find intersect region */
 
-          src =
-              (guint8 *) & (ximagesrc->cursor_image->pixels[((j -
-                          cy) * ximagesrc->cursor_image->width + (i - cx))]);
-          dest =
-              (guint8 *) & (ximage->ximage->data[(j * ximagesrc->width +
-                      i) * (ximagesrc->xcontext->bpp / 8)]);
+          startx = (cx < ximagesrc->startx) ? ximagesrc->startx : cx;
+          starty = (cy < ximagesrc->starty) ? ximagesrc->starty : cy;
+          iwidth = (cx + ximagesrc->cursor_image->width < ximagesrc->endx) ?
+              cx + ximagesrc->cursor_image->width - startx :
+              ximagesrc->endx - startx;
+          iheight = (cy + ximagesrc->cursor_image->height < ximagesrc->endy) ?
+              cy + ximagesrc->cursor_image->height - starty :
+              ximagesrc->endy - starty;
+          clipped = TRUE;
+        }
+      } else {
+        startx = cx;
+        starty = cy;
+        iwidth = ximagesrc->cursor_image->width;
+        iheight = ximagesrc->cursor_image->height;
+      }
 
-          composite_pixel (ximagesrc->xcontext, (guint8 *) dest,
-              (guint8 *) src);
+      if (cursor_in_image) {
+        GST_DEBUG_OBJECT (ximagesrc, "Cursor is in image so trying to draw it");
+        for (i = 0; i < count; i++)
+          ximagesrc->cursor_image->pixels[i] =
+              GUINT_TO_LE (ximagesrc->cursor_image->pixels[i]);
+        /* copy those pixels across */
+        for (j = starty;
+            j < starty + iheight && j < ximagesrc->starty + ximagesrc->height;
+            j++) {
+          for (i = startx;
+              i < startx + iwidth && i < ximagesrc->startx + ximagesrc->width;
+              i++) {
+            guint8 *src, *dest;
+
+            src =
+                (guint8 *) & (ximagesrc->cursor_image->pixels[((j -
+                            cy) * ximagesrc->cursor_image->width + (i - cx))]);
+            dest =
+                (guint8 *) & (ximage->ximage->data[((j -
+                            ximagesrc->starty) * ximagesrc->width + (i -
+                            ximagesrc->startx)) * (ximagesrc->xcontext->bpp /
+                        8)]);
+
+            composite_pixel (ximagesrc->xcontext, (guint8 *) dest,
+                (guint8 *) src);
+          }
         }
       }
     }
   }
 #endif
-
+#ifdef HAVE_XDAMAGE
+  if (ximagesrc->have_xdamage && ximagesrc->use_damage) {
+    /* need to ref ximage to put in last_ximage */
+    gst_buffer_ref (GST_BUFFER (ximage));
+    if (ximagesrc->last_ximage) {
+      gst_buffer_unref (GST_BUFFER (ximagesrc->last_ximage));
+    }
+    ximagesrc->last_ximage = ximage;
+    GST_LOG_OBJECT (ximagesrc, "reffing current buffer for last_ximage");
+  }
+#endif
   return ximage;
 }
 
@@ -535,6 +698,13 @@ gst_ximage_src_create (GstPushSrc * bs, GstBuffer ** buf)
    * before capturing */
 
   GST_OBJECT_LOCK (s);
+  if (GST_ELEMENT_CLOCK (s) == NULL) {
+    GST_OBJECT_UNLOCK (s);
+    GST_ELEMENT_ERROR (s, RESOURCE, FAILED,
+        (_("Cannot operate without a clock")), (NULL));
+    return GST_FLOW_ERROR;
+  }
+
   base_time = GST_ELEMENT_CAST (s)->base_time;
   next_capture_ts = gst_clock_get_time (GST_ELEMENT_CLOCK (s));
   next_capture_ts -= base_time;
@@ -618,6 +788,21 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
     case PROP_SHOW_POINTER:
       src->show_pointer = g_value_get_boolean (value);
       break;
+    case PROP_USE_DAMAGE:
+      src->use_damage = g_value_get_boolean (value);
+      break;
+    case PROP_STARTX:
+      src->startx = g_value_get_uint (value);
+      break;
+    case PROP_STARTY:
+      src->starty = g_value_get_uint (value);
+      break;
+    case PROP_ENDX:
+      src->endx = g_value_get_uint (value);
+      break;
+    case PROP_ENDY:
+      src->endy = g_value_get_uint (value);
+      break;
     default:
       break;
   }
@@ -643,6 +828,21 @@ gst_ximage_src_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_SHOW_POINTER:
       g_value_set_boolean (value, src->show_pointer);
       break;
+    case PROP_USE_DAMAGE:
+      g_value_set_boolean (value, src->use_damage);
+      break;
+    case PROP_STARTX:
+      g_value_set_uint (value, src->startx);
+      break;
+    case PROP_STARTY:
+      g_value_set_uint (value, src->starty);
+      break;
+    case PROP_ENDX:
+      g_value_set_uint (value, src->endx);
+      break;
+    case PROP_ENDY:
+      g_value_set_uint (value, src->endy);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -678,6 +878,8 @@ gst_ximage_src_dispose (GObject * object)
 {
   /* Drop references in the buffer_pool */
   gst_ximage_src_clear_bufpool (GST_XIMAGE_SRC (object));
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -699,8 +901,9 @@ gst_ximage_src_get_caps (GstBaseSrc * bs)
 {
   GstXImageSrc *s = GST_XIMAGE_SRC (bs);
   GstXContext *xcontext;
+  gint x, y, width, height;
 
-  if ((!s->xcontext) && (!gst_ximage_src_open_display (s, NULL)))
+  if ((!s->xcontext) && (!gst_ximage_src_open_display (s, s->display_name)))
     return gst_caps_copy (gst_pad_get_pad_template_caps (GST_BASE_SRC (s)->
             srcpad));
 
@@ -710,6 +913,34 @@ gst_ximage_src_get_caps (GstBaseSrc * bs)
 
   xcontext = s->xcontext;
 
+  x = y = 0;
+  width = xcontext->width;
+  height = xcontext->height;
+  if (s->endx > s->startx && s->endy > s->starty) {
+    /* this means user has put in values */
+    if (s->startx < xcontext->width && s->endx < xcontext->width &&
+        s->starty < xcontext->height && s->endy < xcontext->height) {
+      /* values are fine */
+      x = s->startx;
+      y = s->starty;
+      s->width = width = s->endx - s->startx;
+      s->height = height = s->endy - s->starty;
+    } else {
+      GST_WARNING
+          ("User put in co-ordinates overshooting the X resolution, setting to full screen");
+      s->startx = 0;
+      s->starty = 0;
+      s->endx = 0;
+      s->endy = 0;
+    }
+  } else {
+    GST_WARNING ("User put in bogus co-ordinates, setting to full screen");
+    s->startx = 0;
+    s->starty = 0;
+    s->endx = 0;
+    s->endy = 0;
+  }
+  GST_DEBUG ("width = %d, height=%d", width, height);
   return gst_caps_new_simple ("video/x-raw-rgb",
       "bpp", G_TYPE_INT, xcontext->bpp,
       "depth", G_TYPE_INT, xcontext->depth,
@@ -717,8 +948,8 @@ gst_ximage_src_get_caps (GstBaseSrc * bs)
       "red_mask", G_TYPE_INT, xcontext->r_mask_output,
       "green_mask", G_TYPE_INT, xcontext->g_mask_output,
       "blue_mask", G_TYPE_INT, xcontext->b_mask_output,
-      "width", G_TYPE_INT, xcontext->width,
-      "height", G_TYPE_INT, xcontext->height,
+      "width", G_TYPE_INT, width,
+      "height", G_TYPE_INT, height,
       "framerate", GST_TYPE_FRACTION_RANGE, 1, G_MAXINT, G_MAXINT, 1,
       "pixel-aspect-ratio", GST_TYPE_FRACTION_RANGE, 1, G_MAXINT, G_MAXINT, 1,
       NULL);
@@ -776,15 +1007,74 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
   gc->finalize = gst_ximage_src_finalize;
 
   g_object_class_install_property (gc, PROP_DISPLAY_NAME,
-      g_param_spec_string ("display_name", "Display", "X Display Name", NULL,
+      g_param_spec_string ("display-name", "Display", "X Display Name", NULL,
           G_PARAM_READWRITE));
   g_object_class_install_property (gc, PROP_SCREEN_NUM,
-      g_param_spec_uint ("screen_num", "Screen number", "X Screen Number",
+      g_param_spec_uint ("screen-num", "Screen number", "X Screen Number",
           0, G_MAXINT, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gc, PROP_SHOW_POINTER,
-      g_param_spec_boolean ("show_pointer", "Show Mouse Pointer",
+      g_param_spec_boolean ("show-pointer", "Show Mouse Pointer",
           "Show mouse pointer (if XFixes extension enabled)", TRUE,
           G_PARAM_READWRITE));
+  /**
+   * GstXImageSrc:use-damage
+   *
+   * Use XDamage (if the XDamage extension is enabled)
+   *
+   * Since: 0.10.4
+   **/
+  g_object_class_install_property (gc, PROP_USE_DAMAGE,
+      g_param_spec_boolean ("use-damage", "Use XDamage",
+          "Use XDamage (if XDamage extension enabled)", TRUE,
+          G_PARAM_READWRITE));
+  /**
+   * GstXImageSrc:startx
+   *
+   * X coordinate of top left corner of area to be recorded
+   * (0 for top left of screen)
+   *
+   * Since: 0.10.4
+   **/
+  g_object_class_install_property (gc, PROP_STARTX,
+      g_param_spec_uint ("startx", "Start X co-ordinate",
+          "X coordinate of top left corner of area to be recorded (0 for top left of screen)",
+          0, G_MAXINT, 0, G_PARAM_READWRITE));
+  /**
+   * GstXImageSrc:starty
+   *
+   * Y coordinate of top left corner of area to be recorded
+   * (0 for top left of screen)
+   *
+   * Since: 0.10.4
+   **/
+  g_object_class_install_property (gc, PROP_STARTY,
+      g_param_spec_uint ("starty", "Start Y co-ordinate",
+          "Y coordinate of top left corner of area to be recorded (0 for top left of screen)",
+          0, G_MAXINT, 0, G_PARAM_READWRITE));
+  /**
+   * GstXImageSrc:endx
+   *
+   * X coordinate of bottom right corner of area to be recorded
+   * (0 for bottom right of screen)
+   *
+   * Since: 0.10.4
+   **/
+  g_object_class_install_property (gc, PROP_ENDX,
+      g_param_spec_uint ("endx", "End X",
+          "X coordinate of bottom right corner of area to be recorded (0 for bottom right of screen)",
+          0, G_MAXINT, 0, G_PARAM_READWRITE));
+  /**
+   * GstXImageSrc:endy
+   *
+   * Y coordinate of bottom right corner of area to be recorded
+   * (0 for bottom right of screen)
+   *
+   * Since: 0.10.4
+   **/
+  g_object_class_install_property (gc, PROP_ENDY,
+      g_param_spec_uint ("endy", "End Y",
+          "Y coordinate of bottom right corner of area to be recorded (0 for bottom right of screen)",
+          0, G_MAXINT, 0, G_PARAM_READWRITE));
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -806,6 +1096,11 @@ gst_ximage_src_init (GstXImageSrc * ximagesrc, GstXImageSrcClass * klass)
   ximagesrc->pool_lock = g_mutex_new ();
   ximagesrc->x_lock = g_mutex_new ();
   ximagesrc->show_pointer = TRUE;
+  ximagesrc->use_damage = TRUE;
+  ximagesrc->startx = 0;
+  ximagesrc->starty = 0;
+  ximagesrc->endx = 0;
+  ximagesrc->endy = 0;
 }
 
 static gboolean
